@@ -1,5 +1,7 @@
 /*
- * Copyright 2005-2016 Apple Inc. All rights reserved.
+ * USB backend for macOS.
+ *
+ * Copyright © 2005-2021 Apple Inc. All rights reserved.
  *
  * IMPORTANT:  This Apple software is supplied to you by Apple Computer,
  * Inc. ("Apple") in consideration of your agreement to the following
@@ -288,11 +290,11 @@ static void status_timer_cb(CFRunLoopTimerRef timer, void *info);
 #define IS_64BIT 1
 #define IS_NOT_64BIT 0
 
-#if defined(__i386__) || defined(__x86_64__)
+#if defined(__arm64e__)
 static pid_t	child_pid;		/* Child PID */
-static void run_legacy_backend(int argc, char *argv[], int fd) _CUPS_NORETURN;	/* Starts child backend process running as a ppc executable */
-#endif /* __i386__ || __x86_64__ */
-static void sigterm_handler(int sig);	/* SIGTERM handler */
+static void run_legacy_backend(int argc, char *argv[], int fd) _CUPS_NORETURN;	/* Starts child backend process running as a x86_64 executable */
+static void sigterm_handler(int sig);    /* SIGTERM handler */
+#endif /* __arm64e__ */
 static void sigquit_handler(int sig, siginfo_t *si, void *unused) _CUPS_NORETURN;
 
 #ifdef PARSE_PS_ERRORS
@@ -357,6 +359,8 @@ print_device(const char *uri,		/* I - Device URI */
 
 
   (void)uri;
+  (void)argc;
+  (void)argv;
 
  /*
   * Catch SIGQUIT to determine who is sending it...
@@ -436,18 +440,18 @@ print_device(const char *uri,		/* I - Device URI */
 
     status = registry_open(&driverBundlePath);
 
-#if defined(__i386__) || defined(__x86_64__)
+#if defined(__arm64e__)
     /*
      * If we were unable to load the class drivers for this printer it's
-     * probably because they're ppc or i386. In this case try to run this
-     * backend as i386 or ppc executables so we can use them...
+     * probably because they're x86_64 (or older). In this case try to run this
+     * backend as x86_64 so we can use them...
      */
     if (status == -2)
     {
       run_legacy_backend(argc, argv, print_fd);
       /* Never returns here */
     }
-#endif /* __i386__ || __x86_64__ */
+#endif /* __arm64e__ */
 
     if (status ==  -2)
     {
@@ -768,7 +772,7 @@ print_device(const char *uri,		/* I - Device URI */
 	}
 	else if (bytes > 0)
 	{
-	  fprintf(stderr, "DEBUG: Wrote %d bytes of print data...\n", (int)bytes);
+	  fprintf(stderr, "DEBUG: Wrote %u bytes of print data...\n", (unsigned)bytes);
 
 	  g.print_bytes -= bytes;
 	  print_ptr   += bytes;
@@ -951,7 +955,7 @@ static void *read_thread(void *reference)
       fputs("DEBUG: Got USB return aborted during read\n", stderr);
 
    /*
-    * Make sure this loop executes no more than once every 250 miliseconds...
+    * Make sure this loop executes no more than once every 250 milliseconds...
     */
 
     if ((readstatus != kIOReturnSuccess || rbytes == 0) && (g.wait_eof || !g.read_thread_stop))
@@ -974,7 +978,7 @@ static void *read_thread(void *reference)
          if (readstatus == kIOReturnSuccess && rbytes > 0 && readbuffer[rbytes-1] == 0x4)
            break;
 
-         /* Make sure this loop executes no more than once every 250 miliseconds... */
+         /* Make sure this loop executes no more than once every 250 milliseconds... */
          mach_wait_until(start + delay);
        }
      }
@@ -1124,7 +1128,7 @@ static void iterate_printers(iterator_callback_t callBack, void *userdata)
 
   iterator_reference_t reference = { callBack, userdata, true };
 
-  IONotificationPortRef addNotification = IONotificationPortCreate(kIOMasterPortDefault);
+  IONotificationPortRef addNotification = IONotificationPortCreate(kIOMainPortDefault);
 
   int printingClass = kUSBPrintingClass;
   int printingSubclass = kUSBPrintingSubclass;
@@ -1280,6 +1284,9 @@ static Boolean find_device_cb(io_service_t obj, printer_interface_t printerIntf,
         (*printerIntf)->GetInterfaceNumber(printerIntf, &intfNumber);
         (*printerIntf)->GetLocationID(printerIntf, &intfLocation);
 
+        if (intfProtocol == kUSBPrintingProtocolIPP)
+            return keepLooking;
+
         if (g.serial != NULL && CFStringGetLength(g.serial) > 0)
         {
           if (serial != NULL && CFStringCompare(serial, g.serial, kCFCompareCaseInsensitive) == kCFCompareEqualTo)
@@ -1287,12 +1294,9 @@ static Boolean find_device_cb(io_service_t obj, printer_interface_t printerIntf,
             g.interfaceProtocol = intfProtocol;
             g.location = intfLocation;
             g.alternateSetting = intfAltSetting;
-            if (intfProtocol != kUSBPrintingProtocolIPP)
-            {
-              g.printer_obj = obj;
-              IOObjectRetain(obj);
-            }
-            keepLooking = (intfProtocol == kUSBPrintingProtocolIPP);
+            g.printer_obj = obj;
+            IOObjectRetain(obj);
+            keepLooking = false;
           }
         }
         else
@@ -1300,14 +1304,14 @@ static Boolean find_device_cb(io_service_t obj, printer_interface_t printerIntf,
           if (g.printer_obj != 0)
             IOObjectRelease(g.printer_obj);
 
+            if (g.location == 0 || g.location == intfLocation)
+                keepLooking = false;
+
             g.location = intfLocation;
             g.alternateSetting = intfAltSetting;
             g.interfaceProtocol = intfProtocol;
             g.printer_obj = obj;
             IOObjectRetain(obj);
-
-            if (g.location == 0 || g.location == intfLocation)
-              keepLooking = false;
         }
 
         if (!keepLooking)
@@ -1434,23 +1438,23 @@ static kern_return_t load_classdriver(CFStringRef	    driverPath,
   * Try loading the class driver...
   */
 
-  url = CFURLCreateWithFileSystemPath(NULL, bundle, kCFURLPOSIXPathStyle, true);
+  url = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, bundle, kCFURLPOSIXPathStyle, true);
 
-  if (url)
-  {
-    plugin = CFPlugInCreate(NULL, url);
-    CFRelease(url);
-  }
-  else
-    plugin = NULL;
+  if (url == NULL)
+    return (kr);
+
+  plugin = CFPlugInCreate(kCFAllocatorDefault, url);
+  CFRelease(url);
 
   if (plugin)
   {
     CFArrayRef factories = CFPlugInFindFactoriesForPlugInTypeInPlugIn(kUSBPrinterClassTypeID, plugin);
-    if (factories != NULL && CFArrayGetCount(factories) > 0)
+    if (factories == NULL)
+      return (kr);
+    if (CFArrayGetCount(factories) > 0)
     {
       CFUUIDRef factoryID = CFArrayGetValueAtIndex(factories, 0);
-      IUnknownVTbl **iunknown = CFPlugInInstanceCreate(NULL, factoryID, kUSBPrinterClassTypeID);
+      IUnknownVTbl **iunknown = CFPlugInInstanceCreate(kCFAllocatorDefault, factoryID, kUSBPrinterClassTypeID);
       if (iunknown != NULL)
       {
 	kr = (*iunknown)->QueryInterface(iunknown, CFUUIDGetUUIDBytes(kUSBPrinterClassInterfaceID), (LPVOID *)&driver);
@@ -1472,8 +1476,8 @@ static kern_return_t load_classdriver(CFStringRef	    driverPath,
 	}
 	(*iunknown)->Release(iunknown);
       }
-      CFRelease(factories);
     }
+    CFRelease(factories);
   }
 
   fprintf(stderr, "DEBUG: load_classdriver(%s) (kr:0x%08x)\n", bundlestr, (int)kr);
@@ -1510,12 +1514,11 @@ static kern_return_t load_printerdriver(CFStringRef *driverBundlePath)
   SInt32		score;
   kern_return_t		kr;
   printer_interface_t	interface;
-  HRESULT		res;
 
   kr = IOCreatePlugInInterfaceForService(g.printer_obj, kIOUSBInterfaceUserClientTypeID, kIOCFPlugInInterfaceID, &iodev, &score);
   if (kr == kIOReturnSuccess)
   {
-    if ((res = (*iodev)->QueryInterface(iodev, USB_INTERFACE_KIND, (LPVOID *) &interface)) == noErr)
+    if ((*iodev)->QueryInterface(iodev, USB_INTERFACE_KIND, (LPVOID *) &interface) == noErr)
     {
       *driverBundlePath = IORegistryEntryCreateCFProperty(g.printer_obj, kUSBClassDriverProperty, NULL, kNilOptions);
 
@@ -1589,14 +1592,14 @@ static CFStringRef copy_printer_interface_deviceid(printer_interface_t printer,
 
 			request.wLength = HostToUSBWord(size);
 			request.pData = buffer;
-			berr = (*printer)->ControlRequestTO(printer, (UInt8)0, &request);
+			berr = (*printer)->ControlRequestTO(printer, 0, &request);
 			return berr;
 		};
 
 		/* This request takes the 0 based configuration index. IOKit returns a 1 based configuration index */
 		configurationIndex -= 1;
 
-		bzero(&request, sizeof(request));
+		memset(&request, 0, sizeof(request));
 
 		request.bmRequestType		= USBmakebmRequestType(kUSBIn, kUSBClass, kUSBInterface);
 		request.bRequest			= kUSBPrintClassGetDeviceID;
@@ -1613,14 +1616,14 @@ static CFStringRef copy_printer_interface_deviceid(printer_interface_t printer,
 
 			if (actualLength > 2 && actualLength <= bufferLength - 2)
 			{
-				ret = CFStringCreateWithBytes(NULL, (const UInt8 *) &request.pData[2], actualLength - 2, kCFStringEncodingUTF8, false);
+				ret = CFStringCreateWithBytes(NULL, (const UInt8 *)request.pData + 2, actualLength - 2, kCFStringEncodingUTF8, false);
 			}
 			else if (actualLength > 2) {
 				err = sendRequest(actualLength);
 				if (err == kIOReturnSuccess && request.wLenDone > 0)
 				{
 					actualLength = OSSwapBigToHostInt16(*((UInt16 *)request.pData));
-					ret = CFStringCreateWithBytes(NULL, (const UInt8 *) &request.pData[2], actualLength - 2, kCFStringEncodingUTF8, false);
+					ret = CFStringCreateWithBytes(NULL, (const UInt8 *)request.pData + 2, actualLength - 2, kCFStringEncodingUTF8, false);
 				}
 			}
 		}
@@ -1638,7 +1641,7 @@ static CFStringRef copy_printer_interface_deviceid(printer_interface_t printer,
 		IOUSBDevRequestTO		request;
 		IOUSBDeviceDescriptor	desc;
 
-		bzero(&request, sizeof(request));
+		memset(&request, 0, sizeof(request));
 
 		request.bmRequestType = USBmakebmRequestType( kUSBIn,  kUSBStandard, kUSBDevice );
 		request.bRequest = kUSBRqGetDescriptor;
@@ -1652,7 +1655,7 @@ static CFStringRef copy_printer_interface_deviceid(printer_interface_t printer,
 		err = (*printer)->ControlRequestTO(printer, 0, &request);
 		if (err == kIOReturnSuccess)
 		{
-			CFMutableStringRef extras = CFStringCreateMutable(NULL, 0);
+			CFMutableStringRef extras = CFStringCreateMutable(kCFAllocatorDefault, 0);
 			if (manufacturer == NULL)
 			{
 				manufacturer = copy_printer_interface_indexed_description(printer, desc.iManufacturer, kUSBLanguageEnglish);
@@ -1667,41 +1670,59 @@ static CFStringRef copy_printer_interface_deviceid(printer_interface_t printer,
 					CFStringAppendFormat(extras, NULL, CFSTR("MDL:%@;"), model);
 			}
 
-			if (serial == NULL && desc.iSerialNumber != 0)
+			if (desc.iSerialNumber != 0)
 			{
-				serial = copy_printer_interface_indexed_description(printer, desc.iSerialNumber, kUSBLanguageEnglish);
-				if (serial && CFStringGetLength(serial) > 0)
-					CFStringAppendFormat(extras, NULL, CFSTR("SERN:%@;"), serial);
+				// Always look at the USB serial number since some printers
+				// incorrectly include a bogus static serial number in their
+				// IEEE-1284 device ID string...
+				CFStringRef userial = copy_printer_interface_indexed_description(printer, desc.iSerialNumber, kUSBLanguageEnglish);
+				if (userial && CFStringGetLength(userial) > 0 && (serial == NULL || CFStringCompare(serial, userial, kCFCompareCaseInsensitive) != kCFCompareEqualTo))
+				{
+					if (serial != NULL)
+					{
+						// 1284 serial number doesn't match USB serial number, so  replace the existing SERN: in device ID
+						CFRange range = CFStringFind(ret, serial, 0);
+						CFMutableStringRef deviceIDString = CFStringCreateMutableCopy(kCFAllocatorDefault, 0, ret);
+						CFStringReplace(deviceIDString, range, userial);
+						CFRelease(ret);
+						ret = deviceIDString;
+
+						CFRelease(serial);
+					}
+					else
+					{
+						// No 1284 serial number so add SERN: with USB serial number to device ID
+						CFStringAppendFormat(extras, NULL, CFSTR("SERN:%@;"), userial);
+					}
+					serial = userial;
+				}
+				else if (userial != NULL)
+					CFRelease(userial);
 			}
 
 			if (ret != NULL)
 			{
 				CFStringAppend(extras, ret);
 				CFRelease(ret);
-
-				ret = extras;
-			}
-			else
-			{
-				ret = extras;
 			}
+      ret = extras;
 		}
 	}
 
 	if (ret != NULL)
 	{
-	/* Remove special characters from the serial number */
-	CFRange range = (serial != NULL ? CFStringFind(serial, CFSTR("+"), 0) : CFRangeMake(0, 0));
-	if (range.length == 1)
-	{
-		range = CFStringFind(ret, serial, 0);
+		/* Remove special characters from the serial number */
+		CFRange range = (serial != NULL ? CFStringFind(serial, CFSTR("+"), 0) : CFRangeMake(0, 0));
+		if (range.length == 1)
+		{
+			range = CFStringFind(ret, serial, 0);
 
-		CFMutableStringRef deviceIDString = CFStringCreateMutableCopy(NULL, 0, ret);
-		CFRelease(ret);
+			CFMutableStringRef deviceIDString = CFStringCreateMutableCopy(kCFAllocatorDefault, 0, ret);
+			CFRelease(ret);
 
-		ret = deviceIDString;
-		CFStringFindAndReplace(deviceIDString, CFSTR("+"), CFSTR(""), range, 0);
-	}
+			ret = deviceIDString;
+			CFStringFindAndReplace(deviceIDString, CFSTR("+"), CFSTR(""), range, 0);
+		}
 	}
 
 	if (manufacturer != NULL)
@@ -1728,7 +1749,7 @@ static CFStringRef copy_printer_interface_indexed_description(printer_interface_
 	UInt8 description[256]; // Max possible descriptor length
 	IOUSBDevRequestTO	request;
 
-	bzero(description, 2);
+	description[0] = description[1] = 0;
 
 	request.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBStandard, kUSBDevice);
 	request.bRequest = kUSBRqGetDescriptor;
@@ -1737,12 +1758,12 @@ static CFStringRef copy_printer_interface_indexed_description(printer_interface_
 	request.wLength = 2;
 	request.pData = &description;
 	request.completionTimeout = 0;
-	request.noDataTimeout = 60L;
+	request.noDataTimeout = 60U;
 
 	err = (*printer)->ControlRequestTO(printer, 0, &request);
 	if (err != kIOReturnSuccess && err != kIOReturnOverrun)
 	{
-		bzero(description, request.wLength);
+		memset(description, 0, request.wLength);
 
 		// Let's try again full length. Here's why:
 		//      On USB 2.0 controllers, we will not get an overrun error.  We just get a "babble" error
@@ -1756,16 +1777,16 @@ static CFStringRef copy_printer_interface_indexed_description(printer_interface_
 		request.wLength = sizeof description;
 		request.pData = &description;
 		request.completionTimeout = 0;
-		request.noDataTimeout = 60L;
+		request.noDataTimeout = 60U;
 
 		err = (*printer)->ControlRequestTO(printer, 0, &request);
 		if (err != kIOReturnSuccess && err != kIOReturnUnderrun)
 			return NULL;
 	}
 
-	unsigned int length = description[0];
+	UInt8 length = description[0];
 	if (length == 0)
-		return CFStringCreateWithCString(NULL, "", kCFStringEncodingUTF8);
+		return CFSTR("");
 
 	if (description[1] != kUSBStringDesc)
 		return NULL;
@@ -1775,11 +1796,11 @@ static CFStringRef copy_printer_interface_indexed_description(printer_interface_
 	request.wValue = (kUSBStringDesc << 8) | index;
 	request.wIndex = language;
 
-	bzero(description, length);
+	memset(description, 0, length);
 	request.wLength = (UInt16)length;
 	request.pData = &description;
 	request.completionTimeout = 0;
-	request.noDataTimeout = 60L;
+	request.noDataTimeout = 60U;
 
 	err = (*printer)->ControlRequestTO(printer, 0, &request);
 	if (err != kIOReturnSuccess)
@@ -1788,25 +1809,20 @@ static CFStringRef copy_printer_interface_indexed_description(printer_interface_
 	if (description[1] != kUSBStringDesc)
 		return NULL;
 
-	if ((description[0] & 1) != 0)
-		description[0] &= 0xfe;
+	description[0] &= ~1;
 
-	char buffer[258] = {};
-	unsigned int maxLength = sizeof buffer;
-	if (description[0] > 1)
-	{
-		length = (description[0]-2)/2;
+	if (description[0] < 2)
+		return CFSTR("");
 
-		if (length > maxLength - 1)
-			length = maxLength -1;
+	char buffer[(sizeof(description) - 2) / 2];
+	length = (description[0] - 2) / 2;
 
-		for (unsigned i = 0; i < length; i++)
-			buffer[i] = (char) description[2*i+2];
+	for (UInt8 i = 0; i < length; i++)
+		buffer[i] = (char) description[2 * i + 2];
 
-		buffer[length] = 0;
-	}
+	buffer[length] = 0;
 
-	return CFStringCreateWithCString(NULL, buffer, kCFStringEncodingUTF8);
+	return CFStringCreateWithCString(kCFAllocatorDefault, buffer, kCFStringEncodingUTF8);
 }
 
 /*
@@ -1867,50 +1883,58 @@ static kern_return_t registry_close(void)
  */
 
 static CFStringRef copy_value_for_key(CFStringRef deviceID,
-				      CFStringRef *keys)
+                                      CFStringRef *keys)
 {
-  CFStringRef	value = NULL;
-  CFArrayRef	kvPairs = deviceID != NULL ? CFStringCreateArrayBySeparatingStrings(NULL, deviceID, CFSTR(";")) : NULL;
-  CFIndex	max = kvPairs != NULL ? CFArrayGetCount(kvPairs) : 0;
-  CFIndex	idx = 0;
+  CFStringRef value = NULL; /* Value to return */
+  CFArrayRef kvPairs;       /* pairs derived from separating the device ID*/
+  CFIndex max;              /* The size of the array*/
 
-  while (idx < max && value == NULL)
+  if (deviceID == NULL)
+    return NULL;
+
+  kvPairs = CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault, deviceID, CFSTR(";"));
+  max = CFArrayGetCount(kvPairs);
+
+  for (CFIndex idx = 0; idx < max; idx++)
   {
     CFStringRef kvpair = CFArrayGetValueAtIndex(kvPairs, idx);
     CFIndex idxx = 0;
-    while (keys[idxx] != NULL && value == NULL)
+    for (idxx = 0; keys[idxx] != NULL; idxx++)
     {
       CFRange range = CFStringFind(kvpair, keys[idxx], kCFCompareCaseInsensitive);
-      if (range.length != -1)
+      if (range.length != kCFNotFound)
       {
 	if (range.location != 0)
 	{
-	  CFMutableStringRef theString = CFStringCreateMutableCopy(NULL, 0, kvpair);
+	  CFMutableStringRef theString = CFStringCreateMutableCopy(kCFAllocatorDefault, 0, kvpair);
 	  CFStringTrimWhitespace(theString);
 	  range = CFStringFind(theString, keys[idxx], kCFCompareCaseInsensitive);
 	  if (range.location == 0)
-	    value = CFStringCreateWithSubstring(NULL, theString, CFRangeMake(range.length, CFStringGetLength(theString) - range.length));
+	    value = CFStringCreateWithSubstring(kCFAllocatorDefault, theString, CFRangeMake(range.length, CFStringGetLength(theString) - range.length));
 
 	  CFRelease(theString);
 	}
 	else
 	{
-	  CFStringRef theString = CFStringCreateWithSubstring(NULL, kvpair, CFRangeMake(range.length, CFStringGetLength(kvpair) - range.length));
-	  CFMutableStringRef theString2 = CFStringCreateMutableCopy(NULL, 0, theString);
+	  CFStringRef theString = CFStringCreateWithSubstring(kCFAllocatorDefault, kvpair, CFRangeMake(range.length, CFStringGetLength(kvpair) - range.length));
+	  CFMutableStringRef theString2 = CFStringCreateMutableCopy(kCFAllocatorDefault, 0, theString);
 	  CFRelease(theString);
 
 	  CFStringTrimWhitespace(theString2);
 	  value = theString2;
 	}
       }
-      idxx++;
+
+      if (value != NULL)
+      {
+        CFRelease(kvPairs);
+        return value;
+      }
     }
-    idx++;
   }
 
-  if (kvPairs != NULL)
-    CFRelease(kvPairs);
-  return value;
+  CFRelease(kvPairs);
+  return NULL;
 }
 
 
@@ -1923,9 +1947,9 @@ CFStringRef cfstr_create_trim(const char *cstr)
   CFStringRef		cfstr;
   CFMutableStringRef	cfmutablestr = NULL;
 
-  if ((cfstr = CFStringCreateWithCString(NULL, cstr, kCFStringEncodingUTF8)) != NULL)
+  if ((cfstr = CFStringCreateWithCString(kCFAllocatorDefault, cstr, kCFStringEncodingUTF8)) != NULL)
   {
-    if ((cfmutablestr = CFStringCreateMutableCopy(NULL, 1024, cfstr)) != NULL)
+    if ((cfmutablestr = CFStringCreateMutableCopy(kCFAllocatorDefault, 1024, cfstr)) != NULL)
       CFStringTrimWhitespace(cfmutablestr);
 
     CFRelease(cfstr);
@@ -2009,7 +2033,7 @@ static void parse_options(char *options,
 			     value);
     }
     else if (!_cups_strcasecmp(name, "serial"))
-      strlcpy(serial, value, serial_size);
+      strlcpy(serial, value, (size_t)serial_size);
     else if (!_cups_strcasecmp(name, "location") && location)
       *location = (UInt32)strtoul(value, NULL, 16);
   }
@@ -2030,34 +2054,36 @@ static void parse_options(char *options,
  */
 static void setup_cfLanguage(void)
 {
-  CFStringRef	lang[1] = {NULL};
-  CFArrayRef	langArray = NULL;
-  const char	*requestedLang = NULL;
+  CFStringRef lang[1] = {NULL}; /* StringRef used to create the array */
+  CFArrayRef langArray;      /* The array used to set the language perference */
+  const char *requestedLang; /* The language as retrived from the language
+                                environment variable */
 
   if ((requestedLang = getenv("APPLE_LANGUAGE")) == NULL)
     requestedLang = getenv("LANG");
 
-  if (requestedLang != NULL)
+  if (requestedLang == NULL)
   {
-    lang[0] = CFStringCreateWithCString(kCFAllocatorDefault, requestedLang, kCFStringEncodingUTF8);
-    langArray = CFArrayCreate(kCFAllocatorDefault, (const void **)lang, sizeof(lang) / sizeof(lang[0]), &kCFTypeArrayCallBacks);
-
-    CFPreferencesSetValue(CFSTR("AppleLanguages"), langArray, kCFPreferencesCurrentApplication, kCFPreferencesAnyUser, kCFPreferencesAnyHost);
-    fprintf(stderr, "DEBUG: usb: AppleLanguages=\"%s\"\n", requestedLang);
-
-    CFRelease(lang[0]);
-    CFRelease(langArray);
-  }
-  else
     fputs("DEBUG: usb: LANG and APPLE_LANGUAGE environment variables missing.\n", stderr);
+    return;
+  }
+
+  lang[0] = CFStringCreateWithCString(kCFAllocatorDefault, requestedLang, kCFStringEncodingUTF8);
+  langArray = CFArrayCreate(kCFAllocatorDefault, (const void **)lang, sizeof(lang) / sizeof(lang[0]), &kCFTypeArrayCallBacks);
+
+  CFPreferencesSetValue(CFSTR("AppleLanguages"), langArray, kCFPreferencesCurrentApplication, kCFPreferencesAnyUser, kCFPreferencesAnyHost);
+  fprintf(stderr, "DEBUG: usb: AppleLanguages=\"%s\"\n", requestedLang);
+
+  CFRelease(lang[0]);
+  CFRelease(langArray);
 }
 
 #pragma mark -
-#if defined(__i386__) || defined(__x86_64__)
+#if defined(__arm64e__)
 /*!
  * @function	run_legacy_backend
  *
- * @abstract	Starts child backend process running as a ppc or i386 executable.
+ * @abstract	Starts child backend process running as a x86_64 executable.
  *
  * @result	Never returns; always calls exit().
  *
@@ -2067,7 +2093,7 @@ static void run_legacy_backend(int argc,
 			       char *argv[],
 			       int fd)
 {
-  int	i;
+  size_t i;
   int	exitstatus = 0;
   int	childstatus;
   pid_t	waitpid_status;
@@ -2076,18 +2102,14 @@ static void run_legacy_backend(int argc,
 
 
  /*
-  * If we're running as x86_64 or i386 and couldn't load the class driver
-  * (because it's ppc or i386), then try to re-exec ourselves in ppc or i386
-  * mode to try again. If we don't have a ppc or i386 architecture we may be
+  * If we're running as ARM and couldn't load the class driver
+  * (because it's x86_64, i386 or ppc), then try to re-exec ourselves in x86_64
+  * mode to try again. If we don't have that architecture we may be
   * running with the same architecture again so guard against this by setting
   * and testing an environment variable...
   */
 
-#  ifdef __x86_64__
-  usb_legacy_status = getenv("USB_I386_STATUS");
-#  else
-  usb_legacy_status = getenv("USB_PPC_STATUS");
-#  endif /* __x86_64__ */
+  usb_legacy_status = getenv("USB_LEGACY_STATUS");
 
   if (!usb_legacy_status)
   {
@@ -2116,21 +2138,13 @@ static void run_legacy_backend(int argc,
     * Set the environment variable...
     */
 
-#  ifdef __x86_64__
-    setenv("USB_I386_STATUS", "1", false);
-#  else
-    setenv("USB_PPC_STATUS", "1", false);
-#  endif /* __x86_64__ */
+    setenv("USB_LEGACY_STATUS", "1", false);
 
    /*
     * Tell the kernel to use the specified CPU architecture...
     */
 
-#  ifdef __x86_64__
-    cpu_type_t cpu = CPU_TYPE_I386;
-#  else
-    cpu_type_t cpu = CPU_TYPE_POWERPC;
-#  endif /* __x86_64__ */
+    cpu_type_t cpu = CPU_TYPE_X86_64;
     size_t ocount = 1;
     posix_spawnattr_t attrs;
 
@@ -2139,11 +2153,7 @@ static void run_legacy_backend(int argc,
       posix_spawnattr_setsigdefault(&attrs, &oldmask);
       if (posix_spawnattr_setbinpref_np(&attrs, 1, &cpu, &ocount) || ocount != 1)
       {
-#  ifdef __x86_64__
-	perror("DEBUG: Unable to set binary preference to i386");
-#  else
-	perror("DEBUG: Unable to set binary preference to ppc");
-#  endif /* __x86_64__ */
+	perror("DEBUG: Unable to set binary preference to X86_64");
 	_cupsLangPrintFilter(stderr, "ERROR",
 	                     _("Unable to use legacy USB class driver."));
 	exit(CUPS_BACKEND_STOP);
@@ -2158,7 +2168,7 @@ static void run_legacy_backend(int argc,
       cups_serverbin = CUPS_SERVERBIN;
     snprintf(usbpath, sizeof(usbpath), "%s/backend/usb", cups_serverbin);
 
-    for (i = 0; i < argc && i < (int)(sizeof(my_argv) / sizeof(my_argv[0])) - 1; i ++)
+    for (i = 0; i < argc && i < (sizeof(my_argv) / sizeof(my_argv[0])) - 1; i++)
       my_argv[i] = argv[i];
 
     my_argv[i] = NULL;
@@ -2217,8 +2227,6 @@ static void run_legacy_backend(int argc,
 
   exit(exitstatus);
 }
-#endif /* __i386__ || __x86_64__ */
-
 
 /*
  * 'sigterm_handler()' - SIGTERM handler.
@@ -2227,7 +2235,6 @@ static void run_legacy_backend(int argc,
 static void
 sigterm_handler(int sig)		/* I - Signal */
 {
-#if defined(__i386__) || defined(__x86_64__)
  /*
   * If we started a child process pass the signal on to it...
   */
@@ -2249,12 +2256,12 @@ sigterm_handler(int sig)		/* I - Signal */
       _exit(0);
     else
     {
-      write(2, "DEBUG: Child crashed.\n", 22);
+      backendMessage("DEBUG: Child crashed.\n");
       _exit(CUPS_BACKEND_STOP);
     }
   }
-#endif /* __i386__ || __x86_64__ */
 }
+#endif /* __arm64e__ */
 
 
 /*
@@ -2345,7 +2352,7 @@ static void parse_pserror(char *sockBuffer,
 
       if ((logstrlen = snprintf(logstr, sizeof(logstr), "%s: %s\n", logLevel, pCommentBegin)) >= sizeof(logstr))
       {
-	/* If the string was trucnated make sure it has a linefeed before the nul */
+	/* If the string was truncated make sure it has a linefeed before the nul */
 	logstrlen = sizeof(logstr) - 1;
 	logstr[logstrlen - 1] = '\n';
       }
