@@ -325,7 +325,7 @@ cupsLocalizeDestValue(
 static void
 cups_create_localizations(
     http_t       *http,			/* I - Connection to destination */
-    cups_dinfo_t *dinfo)		/* I - Destination informations */
+    cups_dinfo_t *dinfo)		/* I - Destination information */
 {
   http_t		*http2;		/* Connection for strings file */
   http_status_t		status;		/* Request status */
