@@ -1,16 +1,11 @@
 /*
  * Option encoding routines for CUPS.
  *
- * Copyright 2007-2016 by Apple Inc.
- * Copyright 1997-2007 by Easy Software Products.
+ * Copyright © 2007-2019 by Apple Inc.
+ * Copyright © 1997-2007 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
- *
- * This file is subject to the Apple OS-Developed Software exception.
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -18,6 +13,7 @@
  */
 
 #include "cups-private.h"
+#include "debug-internal.h"
 
 
 /*
@@ -134,6 +130,9 @@ static const _ipp_option_t ipp_options[] =
 							cups_schemes },
   { 1, "finishings",		IPP_TAG_ENUM,		IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
+  { 1, "finishings-col",	IPP_TAG_BEGIN_COLLECTION, IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 1, "finishings-col-default", IPP_TAG_BEGIN_COLLECTION, IPP_TAG_PRINTER },
   { 1, "finishings-default",	IPP_TAG_ENUM,		IPP_TAG_PRINTER },
   { 0, "fit-to-page",		IPP_TAG_BOOLEAN,	IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
@@ -147,6 +146,7 @@ static const _ipp_option_t ipp_options[] =
   { 1, "include-schemes",	IPP_TAG_NAME,		IPP_TAG_OPERATION,
 							IPP_TAG_ZERO,
 							cups_schemes },
+  { 0, "ipp-attribute-fidelity", IPP_TAG_BOOLEAN,	IPP_TAG_OPERATION },
   { 0, "job-account-id",        IPP_TAG_NAME,           IPP_TAG_JOB },
   { 0, "job-account-id-default",IPP_TAG_NAME,           IPP_TAG_PRINTER },
   { 0, "job-accounting-user-id", IPP_TAG_NAME,          IPP_TAG_JOB },
@@ -155,15 +155,20 @@ static const _ipp_option_t ipp_options[] =
   { 0, "job-cancel-after",	IPP_TAG_INTEGER,	IPP_TAG_JOB },
   { 0, "job-cancel-after-default", IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 0, "job-hold-until",	IPP_TAG_KEYWORD,	IPP_TAG_JOB },
+  { 0, "job-hold-until-default", IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
   { 0, "job-id",		IPP_TAG_INTEGER,	IPP_TAG_ZERO }, /* never send as option */
-  { 0, "job-impressions",	IPP_TAG_INTEGER,	IPP_TAG_ZERO }, /* never send as option */
+  { 0, "job-impressions",	IPP_TAG_INTEGER,	IPP_TAG_OPERATION },
   { 0, "job-impressions-completed", IPP_TAG_INTEGER,	IPP_TAG_ZERO }, /* never send as option */
   { 0, "job-k-limit",		IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
-  { 0, "job-k-octets",		IPP_TAG_INTEGER,	IPP_TAG_ZERO }, /* never send as option */
+  { 0, "job-k-octets",		IPP_TAG_INTEGER,	IPP_TAG_OPERATION },
   { 0, "job-k-octets-completed",IPP_TAG_INTEGER,	IPP_TAG_ZERO }, /* never send as option */
-  { 0, "job-media-sheets",	IPP_TAG_INTEGER,	IPP_TAG_ZERO }, /* never send as option */
+  { 0, "job-media-sheets",	IPP_TAG_INTEGER,	IPP_TAG_OPERATION },
   { 0, "job-media-sheets-completed", IPP_TAG_INTEGER,	IPP_TAG_ZERO }, /* never send as option */
+  { 0, "job-name",		IPP_TAG_NAME,		IPP_TAG_OPERATION,
+							IPP_TAG_JOB },
   { 0, "job-page-limit",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
+  { 0, "job-pages",		IPP_TAG_INTEGER,	IPP_TAG_OPERATION },
+  { 0, "job-pages-completed",	IPP_TAG_INTEGER,	IPP_TAG_ZERO }, /* never send as option */
   { 0, "job-password",          IPP_TAG_STRING,         IPP_TAG_OPERATION,
 							IPP_TAG_ZERO,
 							ipp_job_creation },
@@ -171,6 +176,7 @@ static const _ipp_option_t ipp_options[] =
 							IPP_TAG_ZERO,
 							ipp_job_creation },
   { 0, "job-priority",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
+  { 0, "job-priority-default",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 0, "job-quota-period",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 1, "job-sheets",		IPP_TAG_NAME,		IPP_TAG_JOB },
   { 1, "job-sheets-default",	IPP_TAG_NAME,		IPP_TAG_PRINTER },
@@ -189,6 +195,8 @@ static const _ipp_option_t ipp_options[] =
   { 1, "marker-types",		IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
   { 1, "media",			IPP_TAG_KEYWORD,	IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
+  { 0, "media-bottom-margin",	IPP_TAG_INTEGER,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
   { 0, "media-col",		IPP_TAG_BEGIN_COLLECTION, IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
   { 0, "media-col-default",	IPP_TAG_BEGIN_COLLECTION, IPP_TAG_PRINTER },
@@ -197,12 +205,25 @@ static const _ipp_option_t ipp_options[] =
   { 1, "media-default",		IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
   { 0, "media-key",		IPP_TAG_KEYWORD,	IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
+  { 0, "media-left-margin",	IPP_TAG_INTEGER,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "media-right-margin",	IPP_TAG_INTEGER,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
   { 0, "media-size",		IPP_TAG_BEGIN_COLLECTION, IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
+  { 0, "media-size-name",	IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "media-source",		IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "media-top-margin",	IPP_TAG_INTEGER,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
   { 0, "media-type",		IPP_TAG_KEYWORD,	IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
   { 0, "mirror",		IPP_TAG_BOOLEAN,	IPP_TAG_JOB },
   { 0, "mirror-default",	IPP_TAG_BOOLEAN,	IPP_TAG_PRINTER },
+  { 0, "multiple-document-handling", IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "multiple-document-handling-default", IPP_TAG_KEYWORD, IPP_TAG_PRINTER },
   { 0, "natural-scaling",	IPP_TAG_INTEGER,	IPP_TAG_JOB },
   { 0, "natural-scaling-default", IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 0, "notify-charset",	IPP_TAG_CHARSET,	IPP_TAG_SUBSCRIPTION },
@@ -218,18 +239,26 @@ static const _ipp_option_t ipp_options[] =
   { 0, "number-up",		IPP_TAG_INTEGER,	IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
   { 0, "number-up-default",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
+  { 0, "number-up-layout",	IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "number-up-layout-default", IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
   { 0, "orientation-requested",	IPP_TAG_ENUM,		IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
   { 0, "orientation-requested-default", IPP_TAG_ENUM,	IPP_TAG_PRINTER },
+  { 0, "output-bin",		IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "output-bin-default",	IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
   { 1, "overrides",		IPP_TAG_BEGIN_COLLECTION, IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
   { 0, "page-bottom",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
   { 0, "page-bottom-default",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
+  { 0, "page-delivery",		IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "page-delivery-default",	IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
   { 0, "page-left",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
   { 0, "page-left-default",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 1, "page-ranges",		IPP_TAG_RANGE,		IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
-  { 1, "page-ranges-default",	IPP_TAG_RANGE,		IPP_TAG_PRINTER },
   { 0, "page-right",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
   { 0, "page-right-default",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 0, "page-top",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
@@ -270,13 +299,32 @@ static const _ipp_option_t ipp_options[] =
   { 0, "ppi-default",		IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 0, "prettyprint",		IPP_TAG_BOOLEAN,	IPP_TAG_JOB },
   { 0, "prettyprint-default",	IPP_TAG_BOOLEAN,	IPP_TAG_PRINTER },
+  { 0, "print-color-mode",	IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "print-color-mode-default", IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
+  { 0, "print-content-optimize", IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "print-content-optimize-default", IPP_TAG_KEYWORD, IPP_TAG_PRINTER },
   { 0, "print-quality",		IPP_TAG_ENUM,		IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
   { 0, "print-quality-default",	IPP_TAG_ENUM,		IPP_TAG_PRINTER },
+  { 0, "print-rendering-intent", IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "print-rendering-intent-default", IPP_TAG_KEYWORD, IPP_TAG_PRINTER },
+  { 0, "print-scaling",		IPP_TAG_KEYWORD,	IPP_TAG_JOB,
+							IPP_TAG_DOCUMENT },
+  { 0, "print-scaling-default",	IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
+  { 1, "printer-alert",		IPP_TAG_STRING,		IPP_TAG_PRINTER },
+  { 1, "printer-alert-description", IPP_TAG_TEXT,	IPP_TAG_PRINTER },
   { 1, "printer-commands",	IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
   { 0, "printer-error-policy",	IPP_TAG_NAME,		IPP_TAG_PRINTER },
+  { 1, "printer-finisher",	IPP_TAG_STRING,		IPP_TAG_PRINTER },
+  { 1, "printer-finisher-description", IPP_TAG_TEXT,	IPP_TAG_PRINTER },
+  { 1, "printer-finisher-supplies", IPP_TAG_STRING,	IPP_TAG_PRINTER },
+  { 1, "printer-finisher-supplies-description", IPP_TAG_TEXT, IPP_TAG_PRINTER },
   { 0, "printer-geo-location",	IPP_TAG_URI,		IPP_TAG_PRINTER },
   { 0, "printer-info",		IPP_TAG_TEXT,		IPP_TAG_PRINTER },
+  { 1, "printer-input-tray",	IPP_TAG_STRING,		IPP_TAG_PRINTER },
   { 0, "printer-is-accepting-jobs", IPP_TAG_BOOLEAN,	IPP_TAG_PRINTER },
   { 0, "printer-is-shared",	IPP_TAG_BOOLEAN,	IPP_TAG_PRINTER },
   { 0, "printer-is-temporary",	IPP_TAG_BOOLEAN,	IPP_TAG_PRINTER },
@@ -284,11 +332,15 @@ static const _ipp_option_t ipp_options[] =
   { 0, "printer-make-and-model", IPP_TAG_TEXT,		IPP_TAG_PRINTER },
   { 0, "printer-more-info",	IPP_TAG_URI,		IPP_TAG_PRINTER },
   { 0, "printer-op-policy",	IPP_TAG_NAME,		IPP_TAG_PRINTER },
+  { 1, "printer-output-tray",	IPP_TAG_STRING,		IPP_TAG_PRINTER },
   { 0, "printer-resolution",	IPP_TAG_RESOLUTION,	IPP_TAG_JOB,
 							IPP_TAG_DOCUMENT },
+  { 0, "printer-resolution-default", IPP_TAG_RESOLUTION, IPP_TAG_PRINTER },
   { 0, "printer-state",		IPP_TAG_ENUM,		IPP_TAG_PRINTER },
   { 0, "printer-state-change-time", IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { 1, "printer-state-reasons",	IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
+  { 1, "printer-supply",	IPP_TAG_STRING,		IPP_TAG_PRINTER },
+  { 1, "printer-supply-description", IPP_TAG_TEXT,	IPP_TAG_PRINTER },
   { 0, "printer-type",		IPP_TAG_ENUM,		IPP_TAG_PRINTER },
   { 0, "printer-uri",		IPP_TAG_URI,		IPP_TAG_OPERATION },
   { 1, "printer-uri-supported",	IPP_TAG_URI,		IPP_TAG_PRINTER },
@@ -325,16 +377,326 @@ static const _ipp_option_t ipp_options[] =
 static int	compare_ipp_options(_ipp_option_t *a, _ipp_option_t *b);
 
 
+/*
+ * '_cupsEncodeOption()' - Encode a single option as an IPP attribute.
+ */
+
+ipp_attribute_t *			/* O - New attribute or @code NULL@ on error */
+_cupsEncodeOption(
+    ipp_t         *ipp,			/* I - IPP request/response/collection */
+    ipp_tag_t     group_tag,		/* I - Group tag */
+    _ipp_option_t *map,			/* I - Option mapping, if any */
+    const char    *name,		/* I - Attribute name */
+    const char    *value)		/* I - Value */
+{
+  int			i,		/* Looping var */
+			count;		/* Number of values */
+  char			*s,		/* Pointer into option value */
+			*val,		/* Pointer to option value */
+			*copy,		/* Copy of option value */
+			*sep,		/* Option separator */
+			quote;		/* Quote character */
+  ipp_attribute_t	*attr;		/* IPP attribute */
+  ipp_tag_t		value_tag;	/* IPP value tag */
+  ipp_t			*collection;	/* Collection value */
+  int			num_cols;	/* Number of collection values */
+  cups_option_t		*cols;		/* Collection values */
+
+
+  DEBUG_printf(("_cupsEncodeOption(ipp=%p(%s), group=%s, map=%p, name=\"%s\", value=\"%s\")", (void *)ipp, ipp ? ippOpString(ippGetOperation(ipp)) : "", ippTagString(group_tag), (void *)map, name, value));
+
+ /*
+  * Figure out the attribute syntax for encoding...
+  */
+
+  if (!map)
+    map = _ippFindOption(name);
+
+  if (map)
+    value_tag = map->value_tag;
+  else if (!_cups_strcasecmp(value, "true") || !_cups_strcasecmp(value, "false"))
+    value_tag = IPP_TAG_BOOLEAN;
+  else if (value[0] == '{')
+    value_tag = IPP_TAG_BEGIN_COLLECTION;
+  else
+    value_tag = IPP_TAG_NAME;
+
+ /*
+  * Count the number of values...
+  */
+
+  if (map && map->multivalue)
+  {
+    for (count = 1, sep = (char *)value, quote = 0; *sep; sep ++)
+    {
+      if (*sep == quote)
+	quote = 0;
+      else if (!quote && (*sep == '\'' || *sep == '\"'))
+      {
+       /*
+	* Skip quoted option value...
+	*/
+
+	quote = *sep;
+      }
+      else if (*sep == ',' && !quote)
+	count ++;
+      else if (*sep == '\\' && sep[1])
+	sep ++;
+    }
+  }
+  else
+    count = 1;
+
+  DEBUG_printf(("2_cupsEncodeOption: value_tag=%s, count=%d", ippTagString(value_tag), count));
+
+ /*
+  * Allocate memory for the attribute values...
+  */
+
+  if ((attr = ippAddStrings(ipp, group_tag, value_tag, name, count, NULL, NULL)) == NULL)
+  {
+   /*
+    * Ran out of memory!
+    */
+
+    DEBUG_puts("1_cupsEncodeOption: Ran out of memory for attributes.");
+    return (NULL);
+  }
+
+  if (count > 1)
+  {
+   /*
+    * Make a copy of the value we can fiddle with...
+    */
+
+    if ((copy = strdup(value)) == NULL)
+    {
+     /*
+      * Ran out of memory!
+      */
+
+      DEBUG_puts("1_cupsEncodeOption: Ran out of memory for value copy.");
+      ippDeleteAttribute(ipp, attr);
+      return (NULL);
+    }
+
+    val = copy;
+  }
+  else
+  {
+   /*
+    * Since we have a single value, use the value directly...
+    */
+
+    val  = (char *)value;
+    copy = NULL;
+  }
+
+ /*
+  * Scan the value string for values...
+  */
+
+  for (i = 0, sep = val; i < count; val = sep, i ++)
+  {
+   /*
+    * Find the end of this value and mark it if needed...
+    */
+
+    if (count > 1)
+    {
+      for (quote = 0; *sep; sep ++)
+      {
+	if (*sep == quote)
+	{
+	 /*
+	  * Finish quoted value...
+	  */
+
+	  quote = 0;
+	}
+	else if (!quote && (*sep == '\'' || *sep == '\"'))
+	{
+	 /*
+	  * Handle quoted option value...
+	  */
+
+	  quote = *sep;
+	}
+	else if (*sep == ',')
+	  break;
+	else if (*sep == '\\' && sep[1])
+	{
+	 /*
+	  * Skip quoted character...
+	  */
+
+	  memmove(sep, sep + 1, strlen(sep));
+	}
+      }
+
+      if (*sep == ',')
+	*sep++ = '\0';
+    }
+
+   /*
+    * Copy the option value(s) over as needed by the type...
+    */
+
+    switch (attr->value_tag)
+    {
+      case IPP_TAG_INTEGER :
+      case IPP_TAG_ENUM :
+	 /*
+	  * Integer/enumeration value...
+	  */
+
+	  ippSetInteger(ipp, &attr, i, (int)strtol(val, &s, 10));
+	  break;
+
+      case IPP_TAG_BOOLEAN :
+	  if (!_cups_strcasecmp(val, "true") || !_cups_strcasecmp(val, "on") || !_cups_strcasecmp(val, "yes"))
+	  {
+	   /*
+	    * Boolean value - true...
+	    */
+
+            ippSetBoolean(ipp, &attr, i, 1);
+	  }
+	  else
+	  {
+	   /*
+	    * Boolean value - false...
+	    */
+
+            ippSetBoolean(ipp, &attr, i, 0);
+	  }
+	  break;
+
+      case IPP_TAG_RANGE :
+          {
+	   /*
+	    * Range...
+	    */
+
+	    int lower, upper;		/* Lower and upper ranges... */
+
+	    if (*val == '-')
+	    {
+	      lower = 1;
+	      s     = val;
+	    }
+	    else
+	      lower = (int)strtol(val, &s, 10);
+
+	    if (*s == '-')
+	    {
+	      if (s[1])
+		upper = atoi(s + 1);
+	      else
+		upper = 2147483647;
+	    }
+	    else
+	      upper = lower;
+
+            ippSetRange(ipp, &attr, i, lower, upper);
+	  }
+	  break;
+
+      case IPP_TAG_RESOLUTION :
+          {
+	   /*
+	    * Resolution...
+	    */
+	    int		xres, yres;	/* Resolution values */
+	    ipp_res_t	units;		/* Resolution units */
+
+	    xres = (int)strtol(val, &s, 10);
+
+	    if (*s == 'x')
+	      yres = (int)strtol(s + 1, &s, 10);
+	    else
+	      yres = xres;
+
+	    if (!_cups_strcasecmp(s, "dpc") || !_cups_strcasecmp(s, "dpcm"))
+	      units = IPP_RES_PER_CM;
+	    else
+	      units = IPP_RES_PER_INCH;
+
+	    ippSetResolution(ipp, &attr, i, units, xres, yres);
+          }
+	  break;
+
+      case IPP_TAG_STRING :
+	 /*
+	  * octetString
+	  */
+
+          ippSetOctetString(ipp, &attr, i, val, (int)strlen(val));
+	  break;
+
+      case IPP_TAG_BEGIN_COLLECTION :
+	 /*
+	  * Collection value
+	  */
+
+	  num_cols = cupsParseOptions(val, 0, &cols);
+	  if ((collection = ippNew()) == NULL)
+	  {
+	    cupsFreeOptions(num_cols, cols);
+
+	    if (copy)
+	      free(copy);
+
+	    ippDeleteAttribute(ipp, attr);
+	    return (NULL);
+	  }
+
+	  ippSetCollection(ipp, &attr, i, collection);
+	  cupsEncodeOptions2(collection, num_cols, cols, IPP_TAG_JOB);
+	  cupsFreeOptions(num_cols, cols);
+	  ippDelete(collection);
+	  break;
+
+      default :
+	  ippSetString(ipp, &attr, i, val);
+	  break;
+    }
+  }
+
+  if (copy)
+    free(copy);
+
+  return (attr);
+}
+
+
+/*
+ * 'cupsEncodeOption()' - Encode a single option into an IPP attribute.
+ *
+ * @since CUPS 2.3/macOS 10.14@
+ */
+
+ipp_attribute_t	*			/* O - New attribute or @code NULL@ on error */
+cupsEncodeOption(ipp_t      *ipp,	/* I - IPP request/response */
+                 ipp_tag_t  group_tag,	/* I - Attribute group */
+                 const char *name,	/* I - Option name */
+                 const char *value)	/* I - Option string value */
+{
+  return (_cupsEncodeOption(ipp, group_tag, _ippFindOption(name), name, value));
+}
+
+
 /*
  * 'cupsEncodeOptions()' - Encode printer options into IPP attributes.
  *
  * This function adds operation, job, and then subscription attributes,
- * in that order. Use the cupsEncodeOptions2() function to add attributes
+ * in that order. Use the @link cupsEncodeOptions2@ function to add attributes
  * for a single group.
  */
 
 void
-cupsEncodeOptions(ipp_t         *ipp,		/* I - Request to add to */
+cupsEncodeOptions(ipp_t         *ipp,		/* I - IPP request/response */
         	  int           num_options,	/* I - Number of options */
 		  cups_option_t *options)	/* I - Options */
 {
@@ -354,7 +716,7 @@ cupsEncodeOptions(ipp_t         *ipp,		/* I - Request to add to */
  * 'cupsEncodeOptions2()' - Encode printer options into IPP attributes for a group.
  *
  * This function only adds attributes for a single group. Call this
- * function multiple times for each group, or use cupsEncodeOptions()
+ * function multiple times for each group, or use @link cupsEncodeOptions@
  * to add the standard groups.
  *
  * @since CUPS 1.2/macOS 10.5@
@@ -362,24 +724,14 @@ cupsEncodeOptions(ipp_t         *ipp,		/* I - Request to add to */
 
 void
 cupsEncodeOptions2(
-    ipp_t         *ipp,			/* I - Request to add to */
+    ipp_t         *ipp,			/* I - IPP request/response */
     int           num_options,		/* I - Number of options */
     cups_option_t *options,		/* I - Options */
     ipp_tag_t     group_tag)		/* I - Group to encode */
 {
-  int			i, j;		/* Looping vars */
-  int			count;		/* Number of values */
-  char			*s,		/* Pointer into option value */
-			*val,		/* Pointer to option value */
-			*copy,		/* Copy of option value */
-			*sep,		/* Option separator */
-			quote;		/* Quote character */
-  ipp_attribute_t	*attr;		/* IPP attribute */
-  ipp_tag_t		value_tag;	/* IPP value tag */
+  int			i;		/* Looping var */
+  char			*val;		/* Pointer to option value */
   cups_option_t		*option;	/* Current option */
-  ipp_t			*collection;	/* Collection value */
-  int			num_cols;	/* Number of collection values */
-  cups_option_t		*cols;		/* Collection values */
   ipp_op_t		op;		/* Operation for this request */
   const ipp_op_t	*ops;		/* List of allowed operations */
 
@@ -399,24 +751,18 @@ cupsEncodeOptions2(
 
   op = ippGetOperation(ipp);
 
-  if (group_tag == IPP_TAG_OPERATION &&
-      (op == IPP_OP_PRINT_JOB || op == IPP_OP_PRINT_URI ||
-       op == IPP_OP_SEND_DOCUMENT || op == IPP_OP_SEND_URI))
+  if (group_tag == IPP_TAG_OPERATION && (op == IPP_OP_PRINT_JOB || op == IPP_OP_PRINT_URI || op == IPP_OP_SEND_DOCUMENT || op == IPP_OP_SEND_URI))
   {
    /*
     * Handle the document format stuff first...
     */
 
-    if ((val = (char *)cupsGetOption("document-format", num_options,
-                                     options)) != NULL)
-      ippAddString(ipp, IPP_TAG_OPERATION, IPP_TAG_MIMETYPE, "document-format",
-        	   NULL, val);
+    if ((val = (char *)cupsGetOption("document-format", num_options, options)) != NULL)
+      ippAddString(ipp, IPP_TAG_OPERATION, IPP_TAG_MIMETYPE, "document-format", NULL, val);
     else if (cupsGetOption("raw", num_options, options))
-      ippAddString(ipp, IPP_TAG_OPERATION, IPP_TAG_MIMETYPE, "document-format",
-        	   NULL, "application/vnd.cups-raw");
+      ippAddString(ipp, IPP_TAG_OPERATION, IPP_TAG_MIMETYPE, "document-format", NULL, "application/vnd.cups-raw");
     else
-      ippAddString(ipp, IPP_TAG_OPERATION, IPP_TAG_MIMETYPE, "document-format",
-        	   NULL, "application/octet-stream");
+      ippAddString(ipp, IPP_TAG_OPERATION, IPP_TAG_MIMETYPE, "document-format", NULL, "application/octet-stream");
   }
 
  /*
@@ -427,14 +773,11 @@ cupsEncodeOptions2(
   {
     _ipp_option_t	*match;		/* Matching attribute */
 
-
    /*
     * Skip document format options that are handled above...
     */
 
-    if (!_cups_strcasecmp(option->name, "raw") ||
-        !_cups_strcasecmp(option->name, "document-format") ||
-	!option->name[0])
+    if (!_cups_strcasecmp(option->name, "raw") || !_cups_strcasecmp(option->name, "document-format") || !option->name[0])
       continue;
 
    /*
@@ -446,8 +789,6 @@ cupsEncodeOptions2(
       if (match->group_tag != group_tag && match->alt_group_tag != group_tag)
         continue;
 
-      value_tag = match->value_tag;
-
       if (match->operations)
         ops = match->operations;
       else if (group_tag == IPP_TAG_JOB)
@@ -468,12 +809,9 @@ cupsEncodeOptions2(
     {
       int	namelen;		/* Length of name */
 
-
       namelen = (int)strlen(option->name);
 
-      if (namelen < 10 ||
-          (strcmp(option->name + namelen - 8, "-default") &&
-           strcmp(option->name + namelen - 10, "-supported")))
+      if (namelen < 10 || (strcmp(option->name + namelen - 8, "-default") && strcmp(option->name + namelen - 10, "-supported")))
       {
 	if (group_tag != IPP_TAG_JOB && group_tag != IPP_TAG_DOCUMENT)
 	{
@@ -493,12 +831,6 @@ cupsEncodeOptions2(
         ops = ipp_doc_creation;
       else
         ops = ipp_set_printer;
-
-      if (!_cups_strcasecmp(option->value, "true") ||
-          !_cups_strcasecmp(option->value, "false"))
-	value_tag = IPP_TAG_BOOLEAN;
-      else
-	value_tag = IPP_TAG_NAME;
     }
 
    /*
@@ -517,277 +849,7 @@ cupsEncodeOptions2(
       continue;
     }
 
-   /*
-    * Count the number of values...
-    */
-
-    if (match && match->multivalue)
-    {
-      for (count = 1, sep = option->value, quote = 0; *sep; sep ++)
-      {
-	if (*sep == quote)
-	  quote = 0;
-	else if (!quote && (*sep == '\'' || *sep == '\"'))
-	{
-	 /*
-	  * Skip quoted option value...
-	  */
-
-	  quote = *sep++;
-	}
-	else if (*sep == ',' && !quote)
-	  count ++;
-	else if (*sep == '\\' && sep[1])
-	  sep += 2;
-      }
-    }
-    else
-      count = 1;
-
-    DEBUG_printf(("2cupsEncodeOptions2: option=\"%s\", value=\"%s\", count=%d", option->name, option->value, count));
-
-   /*
-    * Allocate memory for the attribute values...
-    */
-
-    if ((attr = ippAddStrings(ipp, group_tag, value_tag, option->name, count,
-                              NULL, NULL)) == NULL)
-    {
-     /*
-      * Ran out of memory!
-      */
-
-      DEBUG_puts("1cupsEncodeOptions2: Ran out of memory for attributes!");
-      return;
-    }
-
-    if (count > 1)
-    {
-     /*
-      * Make a copy of the value we can fiddle with...
-      */
-
-      if ((copy = strdup(option->value)) == NULL)
-      {
-       /*
-	* Ran out of memory!
-	*/
-
-	DEBUG_puts("1cupsEncodeOptions2: Ran out of memory for value copy!");
-	ippDeleteAttribute(ipp, attr);
-	return;
-      }
-
-      val = copy;
-    }
-    else
-    {
-     /*
-      * Since we have a single value, use the value directly...
-      */
-
-      val  = option->value;
-      copy = NULL;
-    }
-
-   /*
-    * Scan the value string for values...
-    */
-
-    for (j = 0, sep = val; j < count; val = sep, j ++)
-    {
-     /*
-      * Find the end of this value and mark it if needed...
-      */
-
-      if (count > 1)
-      {
-	for (quote = 0; *sep; sep ++)
-	{
-	  if (*sep == quote)
-	  {
-	   /*
-	    * Finish quoted value...
-	    */
-
-	    quote = 0;
-	  }
-	  else if (!quote && (*sep == '\'' || *sep == '\"'))
-	  {
-	   /*
-	    * Handle quoted option value...
-	    */
-
-	    quote = *sep;
-	  }
-	  else if (*sep == ',' && count > 1)
-	    break;
-	  else if (*sep == '\\' && sep[1])
-	  {
-	   /*
-	    * Skip quoted character...
-	    */
-
-	    memmove(sep, sep + 1, strlen(sep));
-	    sep ++;
-	  }
-	}
-
-	if (*sep == ',')
-	  *sep++ = '\0';
-      }
-
-     /*
-      * Copy the option value(s) over as needed by the type...
-      */
-
-      switch (attr->value_tag)
-      {
-	case IPP_TAG_INTEGER :
-	case IPP_TAG_ENUM :
-	   /*
-	    * Integer/enumeration value...
-	    */
-
-            attr->values[j].integer = (int)strtol(val, &s, 10);
-
-            DEBUG_printf(("2cupsEncodeOptions2: Added integer option value "
-	                  "%d...", attr->values[j].integer));
-            break;
-
-	case IPP_TAG_BOOLEAN :
-	    if (!_cups_strcasecmp(val, "true") ||
-	        !_cups_strcasecmp(val, "on") ||
-	        !_cups_strcasecmp(val, "yes"))
-	    {
-	     /*
-	      * Boolean value - true...
-	      */
-
-	      attr->values[j].boolean = 1;
-
-              DEBUG_puts("2cupsEncodeOptions2: Added boolean true value...");
-	    }
-	    else
-	    {
-	     /*
-	      * Boolean value - false...
-	      */
-
-	      attr->values[j].boolean = 0;
-
-              DEBUG_puts("2cupsEncodeOptions2: Added boolean false value...");
-	    }
-            break;
-
-	case IPP_TAG_RANGE :
-	   /*
-	    * Range...
-	    */
-
-            if (*val == '-')
-	    {
-	      attr->values[j].range.lower = 1;
-	      s = val;
-	    }
-	    else
-	      attr->values[j].range.lower = (int)strtol(val, &s, 10);
-
-	    if (*s == '-')
-	    {
-	      if (s[1])
-		attr->values[j].range.upper = (int)strtol(s + 1, NULL, 10);
-	      else
-		attr->values[j].range.upper = 2147483647;
-            }
-	    else
-	      attr->values[j].range.upper = attr->values[j].range.lower;
-
-	    DEBUG_printf(("2cupsEncodeOptions2: Added range option value "
-	                  "%d-%d...", attr->values[j].range.lower,
-			  attr->values[j].range.upper));
-            break;
-
-	case IPP_TAG_RESOLUTION :
-	   /*
-	    * Resolution...
-	    */
-
-	    attr->values[j].resolution.xres = (int)strtol(val, &s, 10);
-
-	    if (*s == 'x')
-	      attr->values[j].resolution.yres = (int)strtol(s + 1, &s, 10);
-	    else
-	      attr->values[j].resolution.yres = attr->values[j].resolution.xres;
-
-	    if (!_cups_strcasecmp(s, "dpc") ||
-	        !_cups_strcasecmp(s, "dpcm"))
-              attr->values[j].resolution.units = IPP_RES_PER_CM;
-            else
-              attr->values[j].resolution.units = IPP_RES_PER_INCH;
-
-	    DEBUG_printf(("2cupsEncodeOptions2: Added resolution option value "
-	                  "%s...", val));
-            break;
-
-	case IPP_TAG_STRING :
-           /*
-	    * octet-string
-	    */
-
-            attr->values[j].unknown.length = (int)strlen(val);
-	    attr->values[j].unknown.data   = strdup(val);
-
-            DEBUG_printf(("2cupsEncodeOptions2: Added octet-string value "
-	                  "\"%s\"...", (char *)attr->values[j].unknown.data));
-            break;
-
-        case IPP_TAG_BEGIN_COLLECTION :
-	   /*
-	    * Collection value
-	    */
-
-	    num_cols   = cupsParseOptions(val, 0, &cols);
-	    if ((collection = ippNew()) == NULL)
-	    {
-	      cupsFreeOptions(num_cols, cols);
-
-	      if (copy)
-	        free(copy);
-
-	      ippDeleteAttribute(ipp, attr);
-	      return;
-            }
-
-	    attr->values[j].collection = collection;
-	    cupsEncodeOptions2(collection, num_cols, cols, IPP_TAG_JOB);
-            cupsFreeOptions(num_cols, cols);
-	    break;
-
-	default :
-	    if ((attr->values[j].string.text = _cupsStrAlloc(val)) == NULL)
-	    {
-	     /*
-	      * Ran out of memory!
-	      */
-
-	      DEBUG_puts("1cupsEncodeOptions2: Ran out of memory for string!");
-
-	      if (copy)
-	        free(copy);
-
-	      ippDeleteAttribute(ipp, attr);
-	      return;
-	    }
-
-	    DEBUG_printf(("2cupsEncodeOptions2: Added string value \"%s\"...",
-	                  val));
-            break;
-      }
-    }
-
-    if (copy)
-      free(copy);
+    _cupsEncodeOption(ipp, group_tag, match, option->name, option->value);
   }
 }
 
