@@ -1,6 +1,7 @@
 /*
  * Internet Printing Protocol support functions for CUPS.
  *
+ * Copyright © 2023 by OpenPrinting.
  * Copyright © 2007-2018 by Apple Inc.
  * Copyright © 1997-2007 by Easy Software Products, all rights reserved.
  *
@@ -742,7 +743,7 @@ ippAttributeString(
           {
             unsigned year;		/* Year */
 
-            year = ((unsigned)val->date[0] << 8) + (unsigned)val->date[1];
+            year = ((unsigned)val->date[0] << 8) | (unsigned)val->date[1];
 
 	    if (val->date[9] == 0 && val->date[10] == 0)
 	      snprintf(temp, sizeof(temp), "%04u-%02u-%02uT%02u:%02u:%02uZ",
@@ -1640,6 +1641,7 @@ ippCreateRequestedArray(ipp_t *request)	/* I - IPP request */
     "document-format-default",
     "document-format-details-default",
     "document-format-details-supported",
+    "document-format-preferred",	/* AirPrint extension */
     "document-format-supported",
     "document-format-varying-attributes",
     "document-format-version-default",
@@ -1692,6 +1694,7 @@ ippCreateRequestedArray(ipp_t *request)	/* I - IPP request */
     "marker-types",			/* CUPS extension */
     "member-names",			/* CUPS extension */
     "member-uris",			/* CUPS extension */
+    "mopria-certified",			/* Mopria extension */
     "multiple-destination-uris-supported",/* IPP FaxOut */
     "multiple-document-jobs-supported",
     "multiple-operation-time-out",
@@ -1762,6 +1765,9 @@ ippCreateRequestedArray(ipp_t *request)	/* I - IPP request */
     "printer-state-change-time",
     "printer-state-message",
     "printer-state-reasons",
+    "printer-strings-languages-supported",
+					/* IPP JPS3 */
+    "printer-strings-uri",		/* IPP JPS3 */
     "printer-supply",
     "printer-supply-description",
     "printer-supply-info-uri",
@@ -1769,6 +1775,8 @@ ippCreateRequestedArray(ipp_t *request)	/* I - IPP request */
     "printer-up-time",
     "printer-uri-supported",
     "printer-uuid",
+    "printer-wifi-ssid",		/* AirPrint extension */
+    "printer-wifi-state",		/* AirPrint extension */
     "printer-xri-supported",
     "pwg-raster-document-resolution-supported",
     "pwg-raster-document-sheet-back",
@@ -2262,7 +2270,7 @@ ippErrorString(ipp_status_t error)	/* I - Error status */
   * No, build an "0xxxxx" error string...
   */
 
-  sprintf(cg->ipp_unknown, "0x%04x", error);
+  snprintf(cg->ipp_unknown, sizeof(cg->ipp_unknown), "0x%04x", error);
 
   return (cg->ipp_unknown);
 }
@@ -2339,7 +2347,7 @@ ippOpString(ipp_op_t op)		/* I - Operation ID */
   * No, build an "0xxxxx" operation string...
   */
 
-  sprintf(cg->ipp_unknown, "0x%04x", op);
+  snprintf(cg->ipp_unknown, sizeof(cg->ipp_unknown), "0x%04x", op);
 
   return (cg->ipp_unknown);
 }
