@@ -1,18 +1,13 @@
 /*
  * PPD code emission routines for CUPS.
  *
- * Copyright 2007-2015 by Apple Inc.
+ * Copyright 2007-2019 by Apple Inc.
  * Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  *
  * PostScript is a trademark of Adobe Systems, Inc.
- *
- * This file is subject to the Apple OS-Developed Software exception.
  */
 
 /*
@@ -20,6 +15,7 @@
  */
 
 #include "cups-private.h"
+#include "debug-internal.h"
 #include "ppd.h"
 #if defined(_WIN32) || defined(__EMX__)
 #  include <io.h>
@@ -102,14 +98,14 @@ ppdCollect2(ppd_file_t    *ppd,		/* I - PPD file data */
   */
 
   count = 0;
-  if ((collect = calloc(sizeof(ppd_choice_t *),
-                        (size_t)cupsArrayCount(ppd->marked))) == NULL)
+  if ((collect = calloc((size_t)cupsArrayCount(ppd->marked),
+                        sizeof(ppd_choice_t *))) == NULL)
   {
     *choices = NULL;
     return (0);
   }
 
-  if ((orders = calloc(sizeof(float), (size_t)cupsArrayCount(ppd->marked))) == NULL)
+  if ((orders = calloc((size_t)cupsArrayCount(ppd->marked), sizeof(float))) == NULL)
   {
     *choices = NULL;
     free(collect);
@@ -445,6 +441,9 @@ ppdEmitJCL(ppd_file_t *ppd,		/* I - PPD file record */
     * Clean up the job title...
     */
 
+    if (!title)
+      title = "Unknown";
+
     if ((ptr = strrchr(title, '/')) != NULL)
     {
      /*
@@ -494,6 +493,9 @@ ppdEmitJCL(ppd_file_t *ppd,		/* I - PPD file record */
     * issues with some printer's PJL implementations...
     */
 
+    if (!user)
+      user = "anonymous";
+
     snprintf(displaymsg, sizeof(displaymsg), "%d %s %s", job_id, user, temp);
 
    /*
@@ -845,7 +847,7 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
 	    *bufptr++ = *cptr++;
 	}
       }
-      else
+      else if (choices[i]->code)
       {
        /*
         * Otherwise just copy the option code directly...
@@ -1081,7 +1083,7 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
       DEBUG_printf(("2ppdEmitString: Offset in string is %d...",
                     (int)(bufptr - buffer)));
     }
-    else
+    else if (choices[i]->code)
     {
       strlcpy(bufptr, choices[i]->code, (size_t)(bufend - bufptr + 1));
       bufptr += strlen(bufptr);
