@@ -4,13 +4,8 @@
  * Copyright © 2007-2018 by Apple Inc.
  * Copyright © 1997-2006 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
- *
- * This file is subject to the Apple OS-Developed Software exception.
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -19,6 +14,7 @@
 
 #include "cups-private.h"
 #include "ppd-private.h"
+#include "debug-internal.h"
 #include <fcntl.h>
 #include <sys/stat.h>
 #if defined(_WIN32) || defined(__EMX__)
@@ -34,8 +30,7 @@
 
 static int	cups_get_printer_uri(http_t *http, const char *name,
 		                     char *host, int hostsize, int *port,
-				     char *resource, int resourcesize,
-				     int depth);
+				     char *resource, int resourcesize);
 
 
 /*
@@ -155,18 +150,21 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
 
   if (!name)
   {
+    DEBUG_puts("2cupsGetPPD3: No printer name, returning NULL.");
     _cupsSetError(IPP_STATUS_ERROR_INTERNAL, _("No printer name"), 1);
     return (HTTP_STATUS_NOT_ACCEPTABLE);
   }
 
   if (!modtime)
   {
+    DEBUG_puts("2cupsGetPPD3: No modtime, returning NULL.");
     _cupsSetError(IPP_STATUS_ERROR_INTERNAL, _("No modification time"), 1);
     return (HTTP_STATUS_NOT_ACCEPTABLE);
   }
 
   if (!buffer || bufsize <= 1)
   {
+    DEBUG_puts("2cupsGetPPD3: No filename buffer, returning NULL.");
     _cupsSetError(IPP_STATUS_ERROR_INTERNAL, _("Bad filename buffer"), 1);
     return (HTTP_STATUS_NOT_ACCEPTABLE);
   }
@@ -201,6 +199,8 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
 
       if (buffer[0])
       {
+        DEBUG_printf(("2cupsGetPPD3: Using filename \"%s\".", buffer));
+
         unlink(buffer);
 
 	if (symlink(ppdname, buffer) && errno != EEXIST)
@@ -216,6 +216,7 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
         const char	*tmpdir;	/* TMPDIR environment variable */
 	struct timeval	curtime;	/* Current time */
 
+
 #ifdef __APPLE__
        /*
 	* On macOS and iOS, the TMPDIR environment variable is not always the
@@ -224,17 +225,21 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
 	* per-process TMPDIR value.
 	*/
 
+#ifdef _CS_DARWIN_USER_TEMP_DIR
         char		tmppath[1024];	/* Temporary directory */
+#endif /* _CS_DARWIN_USER_TEMP_DIR */
 
 	if ((tmpdir = getenv("TMPDIR")) != NULL && access(tmpdir, W_OK))
 	  tmpdir = NULL;
 
 	if (!tmpdir)
 	{
+#ifdef _CS_DARWIN_USER_TEMP_DIR
 	  if (confstr(_CS_DARWIN_USER_TEMP_DIR, tmppath, sizeof(tmppath)))
 	    tmpdir = tmppath;
 	  else
-	    tmpdir = "/private/tmp";		/* This should never happen */
+#endif /* _CS_DARWIN_USER_TEMP_DIR */
+	    tmpdir = "/private/tmp";		/* OS X 10.4 and earlier */
 	}
 #else
        /*
@@ -248,6 +253,8 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
 	  tmpdir = "/tmp";
 #endif /* __APPLE__ */
 
+        DEBUG_printf(("2cupsGetPPD3: tmpdir=\"%s\".", tmpdir));
+
        /*
 	* Make the temporary name using the specified directory...
 	*/
@@ -277,12 +284,16 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
 	  if (!symlink(ppdname, buffer))
 	    break;
 
+	  DEBUG_printf(("2cupsGetPPD3: Symlink \"%s\" to \"%s\" failed: %s", ppdname, buffer, strerror(errno)));
+
 	  tries ++;
 	}
 	while (tries < 1000);
 
         if (tries >= 1000)
 	{
+	  DEBUG_puts("2cupsGetPPD3: Unable to symlink after 1000 tries, returning error.");
+
           _cupsSetError(IPP_STATUS_ERROR_INTERNAL, NULL, 0);
 
 	  return (HTTP_STATUS_SERVER_ERROR);
@@ -290,9 +301,13 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
       }
 
       if (*modtime >= ppdinfo.st_mtime)
+      {
+        DEBUG_printf(("2cupsGetPPD3: Returning not-modified, filename=\"%s\".", buffer));
         return (HTTP_STATUS_NOT_MODIFIED);
+      }
       else
       {
+        DEBUG_printf(("2cupsGetPPD3: Returning ok, filename=\"%s\", modtime=%ld.", buffer, (long)ppdinfo.st_mtime));
         *modtime = ppdinfo.st_mtime;
 	return (HTTP_STATUS_OK);
       }
@@ -304,16 +319,24 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
   * Try finding a printer URI for this printer...
   */
 
+  DEBUG_puts("2cupsGetPPD3: Unable to access local file, copying...");
+
   if (!http)
+  {
     if ((http = _cupsConnect()) == NULL)
+    {
+      DEBUG_puts("2cupsGetPPD3: Unable to connect to scheduler.");
       return (HTTP_STATUS_SERVICE_UNAVAILABLE);
+    }
+  }
 
-  if (!cups_get_printer_uri(http, name, hostname, sizeof(hostname), &port,
-                            resource, sizeof(resource), 0))
+  if (!cups_get_printer_uri(http, name, hostname, sizeof(hostname), &port, resource, sizeof(resource)))
+  {
+    DEBUG_puts("2cupsGetPPD3: Unable to get printer URI.");
     return (HTTP_STATUS_NOT_FOUND);
+  }
 
-  DEBUG_printf(("2cupsGetPPD3: Printer hostname=\"%s\", port=%d", hostname,
-                port));
+  DEBUG_printf(("2cupsGetPPD3: Printer hostname=\"%s\", port=%d", hostname, port));
 
   if (cupsServer()[0] == '/' && !_cups_strcasecmp(hostname, "localhost") && port == ippPort())
   {
@@ -357,7 +380,7 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
   else if ((http2 = httpConnect2(hostname, port, NULL, AF_UNSPEC,
 				 cupsEncryption(), 1, 30000, NULL)) == NULL)
   {
-    DEBUG_puts("1cupsGetPPD3: Unable to connect to server");
+    DEBUG_puts("2cupsGetPPD3: Unable to connect to server");
 
     return (HTTP_STATUS_SERVICE_UNAVAILABLE);
   }
@@ -429,7 +452,7 @@ cupsGetPPD3(http_t     *http,		/* I  - HTTP connection or @code CUPS_HTTP_DEFAUL
   * Return the PPD file...
   */
 
-  DEBUG_printf(("1cupsGetPPD3: Returning status %d", status));
+  DEBUG_printf(("2cupsGetPPD3: Returning status %d", status));
 
   return (status);
 }
@@ -525,8 +548,7 @@ cups_get_printer_uri(
     int        hostsize,		/* I - Size of hostname buffer */
     int        *port,			/* O - Port number */
     char       *resource,		/* I - Resource buffer */
-    int        resourcesize,		/* I - Size of resource buffer */
-    int        depth)			/* I - Depth of query */
+    int        resourcesize)		/* I - Size of resource buffer */
 {
   int		i;			/* Looping var */
   ipp_t		*request,		/* IPP request */
@@ -542,7 +564,7 @@ cups_get_printer_uri(
 		};
 
 
-  DEBUG_printf(("4cups_get_printer_uri(http=%p, name=\"%s\", host=%p, hostsize=%d, resource=%p, resourcesize=%d, depth=%d)", http, name, host, hostsize, resource, resourcesize, depth));
+  DEBUG_printf(("4cups_get_printer_uri(http=%p, name=\"%s\", host=%p, hostsize=%d, resource=%p, resourcesize=%d)", http, name, host, hostsize, resource, resourcesize));
 
  /*
   * Setup the printer URI...
