@@ -4,13 +4,8 @@
  * Copyright © 2007-2018 by Apple Inc.
  * Copyright © 1997-2006 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
- *
- * This file is subject to the Apple OS-Developed Software exception.
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -18,13 +13,14 @@
  */
 
 #include "cups-private.h"
+#include "debug-internal.h"
 #include <fcntl.h>
 #include <sys/stat.h>
-#if defined(WIN32) || defined(__EMX__)
+#if defined(_WIN32) || defined(__EMX__)
 #  include <io.h>
 #else
 #  include <unistd.h>
-#endif /* WIN32 || __EMX__ */
+#endif /* _WIN32 || __EMX__ */
 
 
 /*
@@ -912,7 +908,7 @@ cupsStartDocument(
     const char *format,			/* I - MIME type or @code CUPS_FORMAT_foo@ */
     int        last_document)		/* I - 1 for last document in job, 0 otherwise */
 {
-  char		resource[1024],		/* Resource for destinatio */
+  char		resource[1024],		/* Resource for destination */
 		printer_uri[1024];	/* Printer URI */
   ipp_t		*request;		/* Send-Document request */
   http_status_t	status;			/* HTTP status */
