@@ -462,11 +462,10 @@ CompressData(const unsigned char *line,	/* I - Data to compress */
       *comp_ptr++ = temp;
 
      /*
-      * Check the last bit in the current byte and the first bit in the
-      * next byte...
+      * Check the last bit in the current byte...
       */
 
-      if ((temp & 0x01) && comp_ptr < line_end && *comp_ptr & 0x80)
+      if ((temp & 0x01) && comp_ptr < line_end)
         *comp_ptr &= 0x7f;
     }
   }
@@ -955,7 +954,7 @@ OutputRows(
   * Clear the buffer...
   */
 
-  memset(LineBuffers[row], 0, header->cupsWidth * DotBytes);
+  memset(LineBuffers[row], 0, (size_t)header->cupsWidth * (size_t)DotBytes);
 }
 
 
@@ -988,7 +987,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Check command-line...
   */
 
-  if (argc < 6 || argc > 7)
+  if (argc != 6 && argc != 7)
   {
    /*
     * We don't have the correct number of arguments; write an error message
