@@ -332,7 +332,7 @@ StartPage(ppd_file_t         *ppd,	/* I - PPD file */
     Planes[plane] = Planes[0] + plane * header->cupsBytesPerLine / NumPlanes;
 
   if (ColorBits > 1)
-    BitBuffer = malloc(ColorBits * ((header->cupsWidth + 7) / 8));
+    BitBuffer = malloc((size_t)ColorBits * (((size_t)header->cupsWidth + 7) / 8));
   else
     BitBuffer = NULL;
 
@@ -657,7 +657,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Check command-line...
   */
 
-  if (argc < 6 || argc > 7)
+  if (argc != 6 && argc != 7)
   {
    /*
     * We don't have the correct number of arguments; write an error message
@@ -813,8 +813,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   Shutdown();
 
-  if (ppd)
-    ppdClose(ppd);
+  ppdClose(ppd);
 
  /*
   * Close the raster stream...
