@@ -506,7 +506,7 @@ main(int  argc,				/* I - Number of command-line args */
         * Indent...
 	*/
 
-	float amount = 3.0;		/* Indentation */
+	float amount = 3.0f;		/* Indentation */
 
         if (line[3])
           amount = (float)atof(line + 4);
@@ -552,7 +552,7 @@ main(int  argc,				/* I - Number of command-line args */
         * .HP i
 	*/
 
-	float amount = 3.0;		/* Indentation */
+	float amount = 3.0f;		/* Indentation */
 
         if (line[3])
           amount = (float)atof(line + 4);
@@ -585,10 +585,10 @@ main(int  argc,				/* I - Number of command-line args */
         * .TP i
 	*/
 
-	float amount = 3.0;		/* Indentation */
+	float amount = 3.0f;		/* Indentation */
 
         if (line[3])
-          amount = (float)atof(line + 4);
+          amount = (float)atof(line + 4, NULL);
 
 	fputs(end_fonts[font], outfile);
 	font = 0;
@@ -624,7 +624,7 @@ main(int  argc,				/* I - Number of command-line args */
         * .IP x i
 	*/
 
-        float amount = 3.0;		/* Indentation */
+        float amount = 3.0f;		/* Indentation */
         const char *newlist = NULL;	/* New list style */
         const char *newtype = NULL;	/* New list numbering type */
 
@@ -989,7 +989,7 @@ html_alternate(const char *s,		/* I - String */
       link = 0;
     }
 
-    i = 1 - i;
+    i ^= 1;
 
    /*
     * Skip trailing whitespace...
