@@ -1,10 +1,12 @@
 /*
  * Line Printer Daemon interface for CUPS.
  *
- * Copyright 2007-2016 by Apple Inc.
- * Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ * Copyright © 2021-2023 by OpenPrinting.
+ * Copyright © 2007-2016 by Apple Inc.
+ * Copyright © 1997-2006 by Easy Software Products, all rights reserved.
  *
- * Licensed under Apache License v2.0.  See the file "LICENSE" for more information.
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -283,24 +285,19 @@ main(int  argc,				/* I - Number of command-line arguments */
 	break;
 
     case 0x05 : /* Remove jobs */
-        if (list)
-	{
-	 /*
-	  * Grab the agent and skip to the list of users and/or jobs.
-	  */
+       /*
+	* Grab the agent and skip to the list of users and/or jobs.
+	*/
 
-	  agent = list;
+	agent = list;
 
-	  for (; *list && !isspace(*list & 255); list ++);
-	  while (isspace(*list & 255))
-	    *list++ = '\0';
+	for (; *list && !isspace(*list & 255); list ++);
+	while (isspace(*list & 255))
+	  *list++ = '\0';
 
-	  syslog(LOG_INFO, "Remove jobs %s on %s by %s", list, dest, agent);
+	syslog(LOG_INFO, "Remove jobs %s on %s by %s", list, dest, agent);
 
-	  status = (char)remove_jobs(dest, agent, list);
-        }
-	else
-	  status = 1;
+	status = (char)remove_jobs(dest, agent, list);
 
 	putchar(status);
 	break;
@@ -1100,9 +1097,6 @@ recv_print_job(
 		                	    &options);
               break;
 	}
-
-	if (status)
-	  break;
       }
 
      /*
@@ -1268,7 +1262,7 @@ remove_jobs(const char *dest,		/* I - Destination */
 
     request = ippNewRequest(IPP_OP_CANCEL_JOB);
 
-    sprintf(uri, "ipp://localhost/jobs/%d", id);
+    snprintf(uri, sizeof(uri), "ipp://localhost/jobs/%d", id);
     ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "job-uri", NULL, uri);
 
     ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
