@@ -1,14 +1,12 @@
 /*
  * Directory services routines for the CUPS scheduler.
  *
+ * Copyright © 2021-2023 by OpenPrinting.
  * Copyright © 2007-2018 by Apple Inc.
  * Copyright © 1997-2007 by Easy Software Products, all rights reserved.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -18,11 +16,11 @@
 #include "cupsd.h"
 #include <grp.h>
 
-#if defined(HAVE_DNSSD) && defined(__APPLE__)
+#if defined(HAVE_MDNSRESPONDER) && defined(__APPLE__)
 #  include <nameser.h>
 #  include <CoreFoundation/CoreFoundation.h>
 #  include <SystemConfiguration/SystemConfiguration.h>
-#endif /* HAVE_DNSSD && __APPLE__ */
+#endif /* HAVE_MDNSRESPONDER && __APPLE__ */
 
 
 /*
@@ -38,23 +36,14 @@ static int	avahi_running = 0;
  * Local functions...
  */
 
-#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
+#ifdef HAVE_DNSSD
 static char		*get_auth_info_required(cupsd_printer_t *p,
 			                        char *buffer, size_t bufsize);
-#endif /* HAVE_DNSSD || HAVE_AVAHI */
-#ifdef __APPLE__
-static int		get_hostconfig(const char *name);
-#endif /* __APPLE__ */
-static void		update_lpd(int onoff);
-static void		update_smb(int onoff);
-
-
-#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
 #  ifdef __APPLE__
 static void		dnssdAddAlias(const void *key, const void *value,
 			              void *context);
 #  endif /* __APPLE__ */
-static cupsd_txt_t	dnssdBuildTxtRecord(cupsd_printer_t *p, int for_lpd);
+static cupsd_txt_t	dnssdBuildTxtRecord(cupsd_printer_t *p);
 #  ifdef HAVE_AVAHI
 static void		dnssdClientCallback(AvahiClient *c, AvahiClientState state, void *userdata);
 #  endif /* HAVE_AVAHI */
@@ -64,7 +53,7 @@ static void		dnssdDeregisterPrinter(cupsd_printer_t *p, int clear_name, int from
 static const char	*dnssdErrorString(int error);
 static void		dnssdFreeTxtRecord(cupsd_txt_t *txt);
 static void		dnssdRegisterAllPrinters(int from_callback);
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
 static void		dnssdRegisterCallback(DNSServiceRef sdRef,
 					      DNSServiceFlags flags,
 					      DNSServiceErrorType errorCode,
@@ -76,15 +65,15 @@ static void		dnssdRegisterCallback(DNSServiceRef sdRef,
 static void		dnssdRegisterCallback(AvahiEntryGroup *p,
 					      AvahiEntryGroupState state,
 					      void *context);
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 static int		dnssdRegisterInstance(cupsd_srv_t *srv, cupsd_printer_t *p, char *name, const char *type, const char *subtypes, int port, cupsd_txt_t *txt, int commit, int from_callback);
 static void		dnssdRegisterPrinter(cupsd_printer_t *p, int from_callback);
 static void		dnssdStop(void);
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
 static void		dnssdUpdate(void);
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 static void		dnssdUpdateDNSSDName(int from_callback);
-#endif /* HAVE_DNSSD || HAVE_AVAHI */
+#endif /* HAVE_DNSSD */
 
 
 /*
@@ -114,10 +103,10 @@ cupsdDeregisterPrinter(
   * Announce the deletion...
   */
 
-#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
+#ifdef HAVE_DNSSD
   if (removeit && (BrowseLocalProtocols & BROWSE_DNSSD) && DNSSDMaster)
     dnssdDeregisterPrinter(p, 1, 0);
-#endif /* HAVE_DNSSD || HAVE_AVAHI */
+#endif /* HAVE_DNSSD */
 }
 
 
@@ -136,10 +125,10 @@ cupsdRegisterPrinter(cupsd_printer_t *p)/* I - Printer */
       (p->type & (CUPS_PRINTER_REMOTE | CUPS_PRINTER_SCANNER)))
     return;
 
-#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
+#ifdef HAVE_DNSSD
   if ((BrowseLocalProtocols & BROWSE_DNSSD) && DNSSDMaster)
     dnssdRegisterPrinter(p, 0);
-#endif /* HAVE_DNSSD || HAVE_AVAHI */
+#endif /* HAVE_DNSSD */
 }
 
 
@@ -153,10 +142,10 @@ cupsdStartBrowsing(void)
   if (!Browsing || !BrowseLocalProtocols)
     return;
 
-#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
+#ifdef HAVE_DNSSD
   if (BrowseLocalProtocols & BROWSE_DNSSD)
   {
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
     DNSServiceErrorType error;		/* Error from service creation */
 
    /*
@@ -221,27 +210,15 @@ cupsdStartBrowsing(void)
       else
 	avahi_threaded_poll_start(DNSSDMaster);
     }
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
   }
-#endif /* HAVE_DNSSD || HAVE_AVAHI */
 
- /*
-  * Enable LPD and SMB printer sharing as needed through external programs...
-  */
-
-  if (BrowseLocalProtocols & BROWSE_LPD)
-    update_lpd(1);
-
-  if (BrowseLocalProtocols & BROWSE_SMB)
-    update_smb(1);
-
-#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
  /*
   * Register the individual printers
   */
 
   dnssdRegisterAllPrinters(0);
-#endif /* HAVE_DNSSD || HAVE_AVAHI */
+#endif /* HAVE_DNSSD */
 }
 
 
@@ -255,7 +232,7 @@ cupsdStopBrowsing(void)
   if (!Browsing || !BrowseLocalProtocols)
     return;
 
-#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
+#ifdef HAVE_DNSSD
  /*
   * De-register the individual printers
   */
@@ -268,21 +245,11 @@ cupsdStopBrowsing(void)
 
   if ((BrowseLocalProtocols & BROWSE_DNSSD) && DNSSDMaster)
     dnssdStop();
-#endif /* HAVE_DNSSD || HAVE_AVAHI */
-
- /*
-  * Disable LPD and SMB printer sharing as needed through external programs...
-  */
-
-  if (BrowseLocalProtocols & BROWSE_LPD)
-    update_lpd(0);
-
-  if (BrowseLocalProtocols & BROWSE_SMB)
-    update_smb(0);
+#endif /* HAVE_DNSSD */
 }
 
 
-#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
+#ifdef HAVE_DNSSD
 /*
  * 'cupsdUpdateDNSSDName()' - Update the computer name we use for browsing...
  */
@@ -341,23 +308,22 @@ dnssdAddAlias(const void *key,		/* I - Key */
 
 static cupsd_txt_t			/* O - TXT record */
 dnssdBuildTxtRecord(
-    cupsd_printer_t *p,			/* I - Printer information */
-    int             for_lpd)		/* I - 1 = LPD, 0 = IPP */
+    cupsd_printer_t *p)			/* I - Printer information */
 {
   int		i,			/* Looping var */
 		count;			/* Count of key/value pairs */
   char		admin_hostname[256],	/* Hostname for admin page */
 		adminurl_str[256],	/* URL for the admin page */
 		type_str[32],		/* Type to string buffer */
-		state_str[32],		/* State to string buffer */
-		rp_str[1024],		/* Queue name string buffer */
-		air_str[1024],		/* auth-info-required string buffer */
+		rp_str[256],		/* Queue name string buffer */
+		air_str[256],		/* auth-info-required string buffer */
+		urf_str[256],		/* URF string buffer */
 		*keyvalue[32][2],	/* Table of key/value pairs */
                 *ptr;                   /* Pointer in string */
   cupsd_txt_t	txt;			/* TXT record */
   cupsd_listener_t *lis;                /* Current listener */
   const char    *admin_scheme = "http"; /* Admin page URL scheme */
-
+  ipp_attribute_t *urf_supported;	/* urf-supported attribute */
 
  /*
   * Load up the key value pairs...
@@ -365,175 +331,190 @@ dnssdBuildTxtRecord(
 
   count = 0;
 
-  if (!for_lpd || (BrowseLocalProtocols & BROWSE_LPD))
+  keyvalue[count  ][0] = "txtvers";
+  keyvalue[count++][1] = "1";
+
+  keyvalue[count  ][0] = "qtotal";
+  keyvalue[count++][1] = "1";
+
+  keyvalue[count  ][0] = "rp";
+  keyvalue[count++][1] = rp_str;
+  snprintf(rp_str, sizeof(rp_str), "%s/%s", (p->type & CUPS_PRINTER_CLASS) ? "classes" : "printers", p->name);
+
+  keyvalue[count  ][0] = "ty";
+  keyvalue[count++][1] = p->make_model ? p->make_model : "Unknown";
+
+ /*
+  * Get the hostname for the admin page...
+  */
+
+  if (strchr(DNSSDHostName, '.'))
   {
-    keyvalue[count  ][0] = "txtvers";
-    keyvalue[count++][1] = "1";
-
-    keyvalue[count  ][0] = "qtotal";
-    keyvalue[count++][1] = "1";
-
-    keyvalue[count  ][0] = "rp";
-    keyvalue[count++][1] = rp_str;
-    if (for_lpd)
-      strlcpy(rp_str, p->name, sizeof(rp_str));
-    else
-      snprintf(rp_str, sizeof(rp_str), "%s/%s",
-	       (p->type & CUPS_PRINTER_CLASS) ? "classes" : "printers",
-	       p->name);
-
-    keyvalue[count  ][0] = "ty";
-    keyvalue[count++][1] = p->make_model ? p->make_model : "Unknown";
-
    /*
-    * Get the hostname for the admin page...
+    * Use the provided hostname, but make sure it ends with a period...
     */
 
-    if (strchr(DNSSDHostName, '.'))
-    {
-     /*
-      * Use the provided hostname, but make sure it ends with a period...
-      */
-
-      if ((ptr = DNSSDHostName + strlen(DNSSDHostName) - 1) >= DNSSDHostName && *ptr == '.')
-        strlcpy(admin_hostname, DNSSDHostName, sizeof(admin_hostname));
-      else
-        snprintf(admin_hostname, sizeof(admin_hostname), "%s.", DNSSDHostName);
-    }
+    if ((ptr = DNSSDHostName + strlen(DNSSDHostName) - 1) >= DNSSDHostName && *ptr == '.')
+      strlcpy(admin_hostname, DNSSDHostName, sizeof(admin_hostname));
     else
-    {
-     /*
-      * Unqualified hostname gets ".local." added to it...
-      */
-
-      snprintf(admin_hostname, sizeof(admin_hostname), "%s.local.", DNSSDHostName);
-    }
-
-   /*
-    * Get the URL scheme for the admin page...
-    */
-
-#  ifdef HAVE_SSL
-    for (lis = (cupsd_listener_t *)cupsArrayFirst(Listeners); lis; lis = (cupsd_listener_t *)cupsArrayNext(Listeners))
-    {
-      if (lis->encryption != HTTP_ENCRYPTION_NEVER)
-      {
-        admin_scheme = "https";
-        break;
-      }
-    }
-#  endif /* HAVE_SSL */
-
-    httpAssembleURIf(HTTP_URI_CODING_ALL, adminurl_str, sizeof(adminurl_str), admin_scheme,  NULL, admin_hostname, DNSSDPort, "/%s/%s", (p->type & CUPS_PRINTER_CLASS) ? "classes" : "printers", p->name);
-    keyvalue[count  ][0] = "adminurl";
-    keyvalue[count++][1] = adminurl_str;
-
-    if (p->location)
-    {
-      keyvalue[count  ][0] = "note";
-      keyvalue[count++][1] = p->location;
-    }
-
-    keyvalue[count  ][0] = "priority";
-    keyvalue[count++][1] = for_lpd ? "100" : "0";
-
-    keyvalue[count  ][0] = "product";
-    keyvalue[count++][1] = p->pc && p->pc->product ? p->pc->product : "Unknown";
-
-    keyvalue[count  ][0] = "pdl";
-    keyvalue[count++][1] = p->pdl ? p->pdl : "application/postscript";
-
-    if (get_auth_info_required(p, air_str, sizeof(air_str)))
-    {
-      keyvalue[count  ][0] = "air";
-      keyvalue[count++][1] = air_str;
-    }
-
-    keyvalue[count  ][0] = "UUID";
-    keyvalue[count++][1] = p->uuid + 9;
-
-  #ifdef HAVE_SSL
-    keyvalue[count  ][0] = "TLS";
-    keyvalue[count++][1] = "1.2";
-  #endif /* HAVE_SSL */
-
-    if (p->type & CUPS_PRINTER_FAX)
-    {
-      keyvalue[count  ][0] = "Fax";
-      keyvalue[count++][1] = "T";
-      keyvalue[count  ][0] = "rfo";
-      keyvalue[count++][1] = rp_str;
-    }
-
-    if (p->type & CUPS_PRINTER_COLOR)
-    {
-      keyvalue[count  ][0] = "Color";
-      keyvalue[count++][1] = (p->type & CUPS_PRINTER_COLOR) ? "T" : "F";
-    }
-
-    if (p->type & CUPS_PRINTER_DUPLEX)
-    {
-      keyvalue[count  ][0] = "Duplex";
-      keyvalue[count++][1] = (p->type & CUPS_PRINTER_DUPLEX) ? "T" : "F";
-    }
-
-    if (p->type & CUPS_PRINTER_STAPLE)
-    {
-      keyvalue[count  ][0] = "Staple";
-      keyvalue[count++][1] = (p->type & CUPS_PRINTER_STAPLE) ? "T" : "F";
-    }
-
-    if (p->type & CUPS_PRINTER_COPIES)
-    {
-      keyvalue[count  ][0] = "Copies";
-      keyvalue[count++][1] = (p->type & CUPS_PRINTER_COPIES) ? "T" : "F";
-    }
-
-    if (p->type & CUPS_PRINTER_COLLATE)
-    {
-      keyvalue[count  ][0] = "Collate";
-      keyvalue[count++][1] = (p->type & CUPS_PRINTER_COLLATE) ? "T" : "F";
-    }
-
-    if (p->type & CUPS_PRINTER_PUNCH)
-    {
-      keyvalue[count  ][0] = "Punch";
-      keyvalue[count++][1] = (p->type & CUPS_PRINTER_PUNCH) ? "T" : "F";
-    }
-
-    if (p->type & CUPS_PRINTER_BIND)
-    {
-      keyvalue[count  ][0] = "Bind";
-      keyvalue[count++][1] = (p->type & CUPS_PRINTER_BIND) ? "T" : "F";
-    }
-
-    if (p->type & CUPS_PRINTER_SORT)
-    {
-      keyvalue[count  ][0] = "Sort";
-      keyvalue[count++][1] = (p->type & CUPS_PRINTER_SORT) ? "T" : "F";
-    }
-
-    if (p->type & CUPS_PRINTER_MFP)
-    {
-      keyvalue[count  ][0] = "Scan";
-      keyvalue[count++][1] = (p->type & CUPS_PRINTER_MFP) ? "T" : "F";
-    }
-
-    snprintf(type_str, sizeof(type_str), "0x%X", p->type | CUPS_PRINTER_REMOTE);
-    snprintf(state_str, sizeof(state_str), "%d", p->state);
-
-    keyvalue[count  ][0] = "printer-state";
-    keyvalue[count++][1] = state_str;
-
-    keyvalue[count  ][0] = "printer-type";
-    keyvalue[count++][1] = type_str;
+      snprintf(admin_hostname, sizeof(admin_hostname), "%s.", DNSSDHostName);
   }
+  else
+  {
+   /*
+    * Unqualified hostname gets ".local." added to it...
+    */
+
+    snprintf(admin_hostname, sizeof(admin_hostname), "%s.local.", DNSSDHostName);
+  }
+
+ /*
+  * Get the URL scheme for the admin page...
+  */
+
+#  ifdef HAVE_TLS
+  for (lis = (cupsd_listener_t *)cupsArrayFirst(Listeners); lis; lis = (cupsd_listener_t *)cupsArrayNext(Listeners))
+  {
+    if (lis->encryption != HTTP_ENCRYPTION_NEVER)
+    {
+      admin_scheme = "https";
+      break;
+    }
+  }
+#  endif /* HAVE_TLS */
+
+  httpAssembleURIf(HTTP_URI_CODING_ALL, adminurl_str, sizeof(adminurl_str), admin_scheme,  NULL, admin_hostname, DNSSDPort, "/%s/%s", (p->type & CUPS_PRINTER_CLASS) ? "classes" : "printers", p->name);
+  keyvalue[count  ][0] = "adminurl";
+  keyvalue[count++][1] = adminurl_str;
+
+  if (p->location)
+  {
+    keyvalue[count  ][0] = "note";
+    keyvalue[count++][1] = p->location;
+  }
+
+  keyvalue[count  ][0] = "priority";
+  keyvalue[count++][1] = "0";
+
+  keyvalue[count  ][0] = "product";
+  keyvalue[count++][1] = p->pc && p->pc->product ? p->pc->product : "Unknown";
+
+  keyvalue[count  ][0] = "pdl";
+  keyvalue[count++][1] = p->pdl ? p->pdl : "application/postscript";
+
+  if (get_auth_info_required(p, air_str, sizeof(air_str)))
+  {
+    keyvalue[count  ][0] = "air";
+    keyvalue[count++][1] = air_str;
+  }
+
+  keyvalue[count  ][0] = "UUID";
+  keyvalue[count++][1] = p->uuid + 9;
+
+#ifdef HAVE_TLS
+  keyvalue[count  ][0] = "TLS";
+  keyvalue[count++][1] = "1.2";
+#endif /* HAVE_TLS */
+
+  if ((urf_supported = ippFindAttribute(p->ppd_attrs, "urf-supported", IPP_TAG_KEYWORD)) != NULL)
+  {
+    int urf_count = ippGetCount(urf_supported);
+					// Number of URF values
+
+    urf_str[0] = '\0';
+    for (i = 0, ptr = urf_str; i < urf_count; i ++)
+    {
+      const char *value = ippGetString(urf_supported, i, NULL);
+
+      if (ptr > urf_str && ptr < (urf_str + sizeof(urf_str) - 1))
+	*ptr++ = ',';
+
+      strlcpy(ptr, value, sizeof(urf_str) - (size_t)(ptr - urf_str));
+      ptr += strlen(ptr);
+
+      if (ptr >= (urf_str + sizeof(urf_str) - 1))
+	break;
+    }
+
+    keyvalue[count  ][0] = "URF";
+    keyvalue[count++][1] = urf_str;
+  }
+
+  keyvalue[count  ][0] = "mopria-certified";
+  keyvalue[count++][1] = "1.3";
+
+  if (p->type & CUPS_PRINTER_FAX)
+  {
+    keyvalue[count  ][0] = "Fax";
+    keyvalue[count++][1] = "T";
+    keyvalue[count  ][0] = "rfo";
+    keyvalue[count++][1] = rp_str;
+  }
+
+  if (p->type & CUPS_PRINTER_COLOR)
+  {
+    keyvalue[count  ][0] = "Color";
+    keyvalue[count++][1] = (p->type & CUPS_PRINTER_COLOR) ? "T" : "F";
+  }
+
+  if (p->type & CUPS_PRINTER_DUPLEX)
+  {
+    keyvalue[count  ][0] = "Duplex";
+    keyvalue[count++][1] = (p->type & CUPS_PRINTER_DUPLEX) ? "T" : "F";
+  }
+
+  if (p->type & CUPS_PRINTER_STAPLE)
+  {
+    keyvalue[count  ][0] = "Staple";
+    keyvalue[count++][1] = (p->type & CUPS_PRINTER_STAPLE) ? "T" : "F";
+  }
+
+  if (p->type & CUPS_PRINTER_COPIES)
+  {
+    keyvalue[count  ][0] = "Copies";
+    keyvalue[count++][1] = (p->type & CUPS_PRINTER_COPIES) ? "T" : "F";
+  }
+
+  if (p->type & CUPS_PRINTER_COLLATE)
+  {
+    keyvalue[count  ][0] = "Collate";
+    keyvalue[count++][1] = (p->type & CUPS_PRINTER_COLLATE) ? "T" : "F";
+  }
+
+  if (p->type & CUPS_PRINTER_PUNCH)
+  {
+    keyvalue[count  ][0] = "Punch";
+    keyvalue[count++][1] = (p->type & CUPS_PRINTER_PUNCH) ? "T" : "F";
+  }
+
+  if (p->type & CUPS_PRINTER_BIND)
+  {
+    keyvalue[count  ][0] = "Bind";
+    keyvalue[count++][1] = (p->type & CUPS_PRINTER_BIND) ? "T" : "F";
+  }
+
+  if (p->type & CUPS_PRINTER_SORT)
+  {
+    keyvalue[count  ][0] = "Sort";
+    keyvalue[count++][1] = (p->type & CUPS_PRINTER_SORT) ? "T" : "F";
+  }
+
+  if (p->type & CUPS_PRINTER_MFP)
+  {
+    keyvalue[count  ][0] = "Scan";
+    keyvalue[count++][1] = (p->type & CUPS_PRINTER_MFP) ? "T" : "F";
+  }
+
+  snprintf(type_str, sizeof(type_str), "0x%X", p->type | CUPS_PRINTER_REMOTE);
+
+  keyvalue[count  ][0] = "printer-type";
+  keyvalue[count++][1] = type_str;
 
  /*
   * Then pack them into a proper txt record...
   */
 
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
   TXTRecordCreate(&txt, 0, NULL);
 
   for (i = 0; i < count; i ++)
@@ -548,7 +529,7 @@ dnssdBuildTxtRecord(
   for (i = 0, txt = NULL; i < count; i ++)
     txt = avahi_string_list_add_printf(txt, "%s=%s", keyvalue[i][0],
                                        keyvalue[i][1]);
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 
   return (txt);
 }
@@ -693,7 +674,7 @@ dnssdDeregisterInstance(
   if (!srv || !*srv)
     return;
 
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
   (void)from_callback;
 
   DNSServiceRefDeallocate(*srv);
@@ -712,7 +693,7 @@ dnssdDeregisterInstance(
 
   if (!from_callback)
     avahi_threaded_poll_unlock(DNSSDMaster);
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 }
 
 
@@ -735,12 +716,12 @@ dnssdDeregisterPrinter(
   {
     dnssdDeregisterInstance(&p->ipp_srv, from_callback);
 
-#  ifdef HAVE_DNSSD
-#    ifdef HAVE_SSL
+#  ifdef HAVE_MDNSRESPONDER
+#    ifdef HAVE_TLS
     dnssdDeregisterInstance(&p->ipps_srv, from_callback);
-#    endif /* HAVE_SSL */
+#    endif /* HAVE_TLS */
     dnssdDeregisterInstance(&p->printer_srv, from_callback);
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
   }
 
  /*
@@ -766,7 +747,7 @@ dnssdDeregisterPrinter(
 static const char *			/* O - Error message */
 dnssdErrorString(int error)		/* I - Error number */
 {
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
   switch (error)
   {
     case kDNSServiceErr_NoError :
@@ -869,7 +850,7 @@ dnssdErrorString(int error)		/* I - Error number */
 
 #  else /* HAVE_AVAHI */
   return (avahi_strerror(error));
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 }
 
 
@@ -880,13 +861,13 @@ dnssdErrorString(int error)		/* I - Error number */
 static void
 dnssdFreeTxtRecord(cupsd_txt_t *txt)	/* I - TXT record */
 {
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
   TXTRecordDeallocate(txt);
 
 #  else /* HAVE_AVAHI */
   avahi_string_list_free(*txt);
   *txt = NULL;
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 }
 
 
@@ -915,7 +896,7 @@ dnssdRegisterAllPrinters(int from_callback)	/* I - Called from callback? */
  * 'dnssdRegisterCallback()' - DNSServiceRegister callback.
  */
 
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
 static void
 dnssdRegisterCallback(
     DNSServiceRef	sdRef,		/* I - DNS Service reference */
@@ -975,7 +956,7 @@ dnssdRegisterCallback(
 
   /* TODO: Handle collisions with avahi_alternate_service_name(p->reg_name)? */
 }
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 
 
 /*
@@ -999,9 +980,9 @@ dnssdRegisterInstance(
   int	error;				/* Any error */
 
 
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
   (void)from_callback;
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 
   cupsdLogMessage(CUPSD_LOG_DEBUG, "Registering \"%s\" with DNS-SD type \"%s\".", name, type);
 
@@ -1011,22 +992,22 @@ dnssdRegisterInstance(
     * Assign the correct pointer for "srv"...
     */
 
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
     if (!strcmp(type, "_printer._tcp"))
       srv = &p->printer_srv;		/* Target LPD service */
-#    ifdef HAVE_SSL
+#    ifdef HAVE_TLS
     else if (!strcmp(type, "_ipps._tcp"))
       srv = &p->ipps_srv;		/* Target IPPS service */
-#    endif /* HAVE_SSL */
+#    endif /* HAVE_TLS */
     else
       srv = &p->ipp_srv;		/* Target IPP service */
 
 #  else /* HAVE_AVAHI */
     srv = &p->ipp_srv;			/* Target service group */
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
   }
 
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
   (void)commit;
 
 #  else /* HAVE_AVAHI */
@@ -1044,7 +1025,7 @@ dnssdRegisterInstance(
                     name, dnssdErrorString(avahi_client_errno(DNSSDClient)));
     return (0);
   }
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 
  /*
   * Make sure the name is <= 63 octets, and when we truncate be sure to
@@ -1068,7 +1049,7 @@ dnssdRegisterInstance(
   * Register the service...
   */
 
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
   if (subtypes)
     snprintf(temp, sizeof(temp), "%s,%s", type, subtypes);
   else
@@ -1076,7 +1057,7 @@ dnssdRegisterInstance(
 
   *srv  = DNSSDMaster;
   error = DNSServiceRegister(srv, kDNSServiceFlagsShareConnection,
-			     0, name, temp, NULL, NULL, htons(port),
+			     0, name, temp, NULL, DNSSDHostName, htons(port),
 			     txt ? TXTRecordGetLength(txt) : 0,
 			     txt ? TXTRecordGetBytesPtr(txt) : NULL,
 			     dnssdRegisterCallback, p);
@@ -1091,7 +1072,7 @@ dnssdRegisterInstance(
 
   error = avahi_entry_group_add_service_strlst(*srv, AVAHI_IF_UNSPEC,
                                                AVAHI_PROTO_UNSPEC, 0, name,
-                                               type, NULL, NULL, port,
+                                               type, NULL, DNSSDHostName, port,
                                                txt ? *txt : NULL);
   if (error)
     cupsdLogMessage(CUPSD_LOG_DEBUG, "DNS-SD service add for \"%s\" failed.",
@@ -1157,7 +1138,7 @@ dnssdRegisterInstance(
 
   if (!from_callback)
     avahi_threaded_poll_unlock(DNSSDMaster);
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 
   if (error)
   {
@@ -1183,10 +1164,8 @@ dnssdRegisterPrinter(
     int             from_callback)	/* I - Called from callback? */
 {
   char		name[256];		/* Service name */
-  int		printer_port;		/* LPD port number */
   int		status;			/* Registration status */
-  cupsd_txt_t	ipp_txt,		/* IPP(S) TXT record */
- 		printer_txt;		/* LPD TXT record */
+  cupsd_txt_t	ipp_txt;		/* IPP(S) TXT record */
 
 
   cupsdLogMessage(CUPSD_LOG_DEBUG2, "dnssdRegisterPrinter(%s) %s", p->name,
@@ -1233,24 +1212,17 @@ dnssdRegisterPrinter(
   * Register IPP and LPD...
   *
   * We always must register the "_printer" service type in order to reserve
-  * our name, but use port number 0 if we haven't actually configured cups-lpd
-  * to share via LPD...
+  * our name, but use port number 0 so that we don't have clients using LPD...
   */
 
-  ipp_txt     = dnssdBuildTxtRecord(p, 0);
-  printer_txt = dnssdBuildTxtRecord(p, 1);
+  ipp_txt = dnssdBuildTxtRecord(p);
 
-  if (BrowseLocalProtocols & BROWSE_LPD)
-    printer_port = 515;
-  else
-    printer_port = 0;
+  status = dnssdRegisterInstance(NULL, p, name, "_printer._tcp", NULL, 0, NULL, 0, from_callback);
 
-  status = dnssdRegisterInstance(NULL, p, name, "_printer._tcp", NULL, printer_port, &printer_txt, 0, from_callback);
-
-#  ifdef HAVE_SSL
+#  ifdef HAVE_TLS
   if (status)
     dnssdRegisterInstance(NULL, p, name, "_ipps._tcp", DNSSDSubTypes, DNSSDPort, &ipp_txt, 0, from_callback);
-#  endif /* HAVE_SSL */
+#  endif /* HAVE_TLS */
 
   if (status)
   {
@@ -1265,7 +1237,6 @@ dnssdRegisterPrinter(
   }
 
   dnssdFreeTxtRecord(&ipp_txt);
-  dnssdFreeTxtRecord(&printer_txt);
 
   if (status)
   {
@@ -1285,12 +1256,12 @@ dnssdRegisterPrinter(
 
     dnssdDeregisterInstance(&p->ipp_srv, from_callback);
 
-#  ifdef HAVE_DNSSD
-#    ifdef HAVE_SSL
+#  ifdef HAVE_MDNSRESPONDER
+#    ifdef HAVE_TLS
     dnssdDeregisterInstance(&p->ipps_srv, from_callback);
-#    endif /* HAVE_SSL */
+#    endif /* HAVE_TLS */
     dnssdDeregisterInstance(&p->printer_srv, from_callback);
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
   }
 }
 
@@ -1320,7 +1291,7 @@ dnssdStop(void)
 
   dnssdDeregisterInstance(&WebIFSrv, 0);
 
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
   cupsdRemoveSelect(DNSServiceRefSockFD(DNSSDMaster));
 
   DNSServiceRefDeallocate(DNSSDMaster);
@@ -1341,7 +1312,7 @@ dnssdStop(void)
     avahi_threaded_poll_free(DNSSDMaster);
     DNSSDMaster = NULL;
   }
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 
   cupsArrayDelete(DNSSDPrinters);
   DNSSDPrinters = NULL;
@@ -1350,7 +1321,7 @@ dnssdStop(void)
 }
 
 
-#  ifdef HAVE_DNSSD
+#  ifdef HAVE_MDNSRESPONDER
 /*
  * 'dnssdUpdate()' - Handle DNS-SD queries.
  */
@@ -1369,7 +1340,7 @@ dnssdUpdate(void)
     dnssdStop();
   }
 }
-#  endif /* HAVE_DNSSD */
+#  endif /* HAVE_MDNSRESPONDER */
 
 
 /*
@@ -1457,23 +1428,29 @@ dnssdUpdateDNSSDName(int from_callback)	/* I - Called from callback? */
       cupsdSetString(&DNSSDComputerName, ServerName);
     }
 
-   /*
-    * Get the local hostname from the dynamic store...
-    */
-
-    cupsdClearString(&DNSSDHostName);
-
-    if ((nameRef = SCDynamicStoreCopyLocalHostName(sc)) != NULL)
+    if (!DNSSDHostName)
     {
-      if (CFStringGetCString(nameRef, nameBuffer, sizeof(nameBuffer),
-			     kCFStringEncodingUTF8))
-      {
-        cupsdLogMessage(CUPSD_LOG_DEBUG,
-	                "Dynamic store host name is \"%s\".", nameBuffer);
-	cupsdSetString(&DNSSDHostName, nameBuffer);
-      }
+     /*
+      * Get the local hostname from the dynamic store...
+      */
 
-      CFRelease(nameRef);
+      if ((nameRef = SCDynamicStoreCopyLocalHostName(sc)) != NULL)
+      {
+	if (CFStringGetCString(nameRef, nameBuffer, sizeof(nameBuffer),
+			       kCFStringEncodingUTF8))
+	{
+	  cupsdLogMessage(CUPSD_LOG_DEBUG, "Dynamic store host name is \"%s\".", nameBuffer);
+
+	  if (strchr(nameBuffer, '.'))
+	    cupsdSetString(&DNSSDHostName, nameBuffer);
+	  else
+	    cupsdSetStringf(&DNSSDHostName, "%s.local", nameBuffer);
+
+	  cupsdLogMessage(CUPSD_LOG_INFO, "Defaulting to \"DNSSDHostName %s\".", DNSSDHostName);
+	}
+
+	CFRelease(nameRef);
+      }
     }
 
     if (!DNSSDHostName)
@@ -1482,9 +1459,10 @@ dnssdUpdateDNSSDName(int from_callback)	/* I - Called from callback? */
       * Use the ServerName instead...
       */
 
-      cupsdLogMessage(CUPSD_LOG_DEBUG,
-                      "Using ServerName \"%s\" as host name.", ServerName);
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Using ServerName \"%s\" as host name.", ServerName);
       cupsdSetString(&DNSSDHostName, ServerName);
+
+      cupsdLogMessage(CUPSD_LOG_INFO, "Defaulting to \"DNSSDHostName %s\".", DNSSDHostName);
     }
 
    /*
@@ -1518,26 +1496,37 @@ dnssdUpdateDNSSDName(int from_callback)	/* I - Called from callback? */
   if (DNSSDClient)
   {
     const char	*host_name = avahi_client_get_host_name(DNSSDClient);
-    const char	*host_fqdn = avahi_client_get_host_name_fqdn(DNSSDClient);
 
     cupsdSetString(&DNSSDComputerName, host_name ? host_name : ServerName);
 
-    if (host_fqdn)
-      cupsdSetString(&DNSSDHostName, host_fqdn);
-    else if (strchr(ServerName, '.'))
-      cupsdSetString(&DNSSDHostName, ServerName);
-    else
-      cupsdSetStringf(&DNSSDHostName, "%s.local", ServerName);
+    if (!DNSSDHostName)
+    {
+      const char *host_fqdn = avahi_client_get_host_name_fqdn(DNSSDClient);
+
+      if (host_fqdn)
+	cupsdSetString(&DNSSDHostName, host_fqdn);
+      else if (strchr(ServerName, '.'))
+	cupsdSetString(&DNSSDHostName, ServerName);
+      else
+	cupsdSetStringf(&DNSSDHostName, "%s.local", ServerName);
+
+      cupsdLogMessage(CUPSD_LOG_INFO, "Defaulting to \"DNSSDHostName %s\".", DNSSDHostName);
+    }
   }
   else
 #  endif /* HAVE_AVAHI */
   {
     cupsdSetString(&DNSSDComputerName, ServerName);
 
-    if (strchr(ServerName, '.'))
-      cupsdSetString(&DNSSDHostName, ServerName);
-    else
-      cupsdSetStringf(&DNSSDHostName, "%s.local", ServerName);
+    if (!DNSSDHostName)
+    {
+      if (strchr(ServerName, '.'))
+	cupsdSetString(&DNSSDHostName, ServerName);
+      else
+	cupsdSetStringf(&DNSSDHostName, "%s.local", ServerName);
+
+      cupsdLogMessage(CUPSD_LOG_INFO, "Defaulting to \"DNSSDHostName %s\".", DNSSDHostName);
+    }
   }
 
  /*
@@ -1634,220 +1623,4 @@ get_auth_info_required(
 
   return ("none");
 }
-#endif /* HAVE_DNSSD || HAVE_AVAHI */
-
-
-#ifdef __APPLE__
-/*
- * 'get_hostconfig()' - Get an /etc/hostconfig service setting.
- */
-
-static int				/* O - 1 for YES or AUTOMATIC, 0 for NO */
-get_hostconfig(const char *name)	/* I - Name of service */
-{
-  cups_file_t	*fp;			/* Hostconfig file */
-  char		line[1024],		/* Line from file */
-		*ptr;			/* Pointer to value */
-  int		state = 1;		/* State of service */
-
-
- /*
-  * Try opening the /etc/hostconfig file; if we can't open it, assume that
-  * the service is enabled/auto.
-  */
-
-  if ((fp = cupsFileOpen("/etc/hostconfig", "r")) != NULL)
-  {
-   /*
-    * Read lines from the file until we find the service...
-    */
-
-    while (cupsFileGets(fp, line, sizeof(line)))
-    {
-      if (line[0] == '#' || (ptr = strchr(line, '=')) == NULL)
-        continue;
-
-      *ptr++ = '\0';
-
-      if (!_cups_strcasecmp(line, name))
-      {
-       /*
-        * Found the service, see if it is set to "-NO-"...
-	*/
-
-	if (!_cups_strncasecmp(ptr, "-NO-", 4))
-	  state = 0;
-        break;
-      }
-    }
-
-    cupsFileClose(fp);
-  }
-
-  return (state);
-}
-#endif /* __APPLE__ */
-
-
-/*
- * 'update_lpd()' - Update the LPD configuration as needed.
- */
-
-static void
-update_lpd(int onoff)			/* - 1 = turn on, 0 = turn off */
-{
-  if (!LPDConfigFile)
-    return;
-
-#ifdef __APPLE__
- /*
-  * Allow /etc/hostconfig CUPS_LPD service setting to override cupsd.conf
-  * setting for backwards-compatibility.
-  */
-
-  if (onoff && !get_hostconfig("CUPS_LPD"))
-    onoff = 0;
-#endif /* __APPLE__ */
-
-  if (!strncmp(LPDConfigFile, "xinetd:///", 10))
-  {
-   /*
-    * Enable/disable LPD via the xinetd.d config file for cups-lpd...
-    */
-
-    char	newfile[1024];		/* New cups-lpd.N file */
-    cups_file_t	*ofp,			/* Original file pointer */
-		*nfp;			/* New file pointer */
-    char	line[1024];		/* Line from file */
-
-
-    snprintf(newfile, sizeof(newfile), "%s.N", LPDConfigFile + 9);
-
-    if ((ofp = cupsFileOpen(LPDConfigFile + 9, "r")) == NULL)
-    {
-      cupsdLogMessage(CUPSD_LOG_ERROR, "Unable to open \"%s\" - %s",
-                      LPDConfigFile + 9, strerror(errno));
-      return;
-    }
-
-    if ((nfp = cupsFileOpen(newfile, "w")) == NULL)
-    {
-      cupsdLogMessage(CUPSD_LOG_ERROR, "Unable to create \"%s\" - %s",
-                      newfile, strerror(errno));
-      cupsFileClose(ofp);
-      return;
-    }
-
-   /*
-    * Copy all of the lines from the cups-lpd file...
-    */
-
-    while (cupsFileGets(ofp, line, sizeof(line)))
-    {
-      if (line[0] == '{')
-      {
-        cupsFilePrintf(nfp, "%s\n", line);
-        snprintf(line, sizeof(line), "\tdisable = %s",
-	         onoff ? "no" : "yes");
-      }
-      else if (!strstr(line, "disable ="))
-        cupsFilePrintf(nfp, "%s\n", line);
-    }
-
-    cupsFileClose(nfp);
-    cupsFileClose(ofp);
-    rename(newfile, LPDConfigFile + 9);
-  }
-#ifdef __APPLE__
-  else if (!strncmp(LPDConfigFile, "launchd:///", 11))
-  {
-   /*
-    * Enable/disable LPD via the launchctl command...
-    */
-
-    char	*argv[5],		/* Arguments for command */
-		*envp[MAX_ENV];		/* Environment for command */
-    int		pid;			/* Process ID */
-
-
-    cupsdLoadEnv(envp, (int)(sizeof(envp) / sizeof(envp[0])));
-    argv[0] = (char *)"launchctl";
-    argv[1] = (char *)(onoff ? "load" : "unload");
-    argv[2] = (char *)"-w";
-    argv[3] = LPDConfigFile + 10;
-    argv[4] = NULL;
-
-    cupsdStartProcess("/bin/launchctl", argv, envp, -1, -1, -1, -1, -1, 1,
-                      NULL, NULL, &pid);
-  }
-#endif /* __APPLE__ */
-  else
-    cupsdLogMessage(CUPSD_LOG_INFO, "Unknown LPDConfigFile scheme!");
-}
-
-
-/*
- * 'update_smb()' - Update the SMB configuration as needed.
- */
-
-static void
-update_smb(int onoff)			/* I - 1 = turn on, 0 = turn off */
-{
-  if (!SMBConfigFile)
-    return;
-
-  if (!strncmp(SMBConfigFile, "samba:///", 9))
-  {
-   /*
-    * Enable/disable SMB via the specified smb.conf config file...
-    */
-
-    char	newfile[1024];		/* New smb.conf.N file */
-    cups_file_t	*ofp,			/* Original file pointer */
-		*nfp;			/* New file pointer */
-    char	line[1024];		/* Line from file */
-    int		in_printers;		/* In [printers] section? */
-
-
-    snprintf(newfile, sizeof(newfile), "%s.N", SMBConfigFile + 8);
-
-    if ((ofp = cupsFileOpen(SMBConfigFile + 8, "r")) == NULL)
-    {
-      cupsdLogMessage(CUPSD_LOG_ERROR, "Unable to open \"%s\" - %s",
-                      SMBConfigFile + 8, strerror(errno));
-      return;
-    }
-
-    if ((nfp = cupsFileOpen(newfile, "w")) == NULL)
-    {
-      cupsdLogMessage(CUPSD_LOG_ERROR, "Unable to create \"%s\" - %s",
-                      newfile, strerror(errno));
-      cupsFileClose(ofp);
-      return;
-    }
-
-   /*
-    * Copy all of the lines from the smb.conf file...
-    */
-
-    in_printers = 0;
-
-    while (cupsFileGets(ofp, line, sizeof(line)))
-    {
-      if (in_printers && strstr(line, "printable ="))
-        snprintf(line, sizeof(line), "    printable = %s",
-	         onoff ? "yes" : "no");
-
-      cupsFilePrintf(nfp, "%s\n", line);
-
-      if (line[0] == '[')
-        in_printers = !strcmp(line, "[printers]");
-    }
-
-    cupsFileClose(nfp);
-    cupsFileClose(ofp);
-    rename(newfile, SMBConfigFile + 8);
-  }
-  else
-    cupsdLogMessage(CUPSD_LOG_INFO, "Unknown SMBConfigFile scheme!");
-}
+#endif /* HAVE_DNSSD */
