@@ -4,11 +4,7 @@
  * Copyright 2007-2016 by Apple Inc.
  * Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more information.
  */
 
 /*
@@ -124,7 +120,7 @@ cupsdStartListening(void)
 {
   int			p;		/* Port number */
   cupsd_listener_t	*lis;		/* Current listening socket */
-  char			s[256];		/* String addresss */
+  char			s[256];		/* String address */
   const char		*have_domain;	/* Have a domain socket? */
   static const char * const encryptions[] =
 		{			/* Encryption values */
@@ -164,7 +160,7 @@ cupsdStartListening(void)
 
       if (lis->fd == -1)
       {
-	cupsdLogMessage(CUPSD_LOG_ERROR,
+	cupsdLogMessage(errno == EAFNOSUPPORT ? CUPSD_LOG_INFO : CUPSD_LOG_ERROR,
 			"Unable to open listen socket for address %s:%d - %s.",
 			s, p, strerror(errno));
 
