@@ -4,11 +4,7 @@
  * Copyright 2007-2013 by Apple Inc.
  * Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * file is missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more information.
  */
 
 #ifndef _CUPS_MIME_H_
@@ -17,6 +13,7 @@
 #  include <cups/array.h>
 #  include <cups/ipp.h>
 #  include <cups/file.h>
+#  include <cups/thread-private.h>
 #  include <regex.h>
 
 
@@ -110,6 +107,7 @@ typedef struct _mime_s			/**** MIME Database ****/
   cups_array_t		*srcs;		/* Filters sorted by source type */
   mime_error_cb_t	error_cb;	/* Error message callback */
   void			*error_ctx;	/* Pointer for callback */
+  _cups_rwlock_t	lock;	/* Read/write lock for guarding data for background updates */
 } mime_t;
 
 
