@@ -4,11 +4,7 @@
  * Copyright 2007-2011, 2014 by Apple Inc.
  * Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * file is missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more information.
  */
 
 /*
@@ -120,7 +116,7 @@ cupsdCheckPolicy(cupsd_policy_t *p,	/* I - Policy */
   {
     cupsdLogMessage(CUPSD_LOG_CRIT, "cupsdCheckPolicy: p=%p, con=%p.", p, con);
 
-    return ((http_status_t)0);
+    return (HTTP_STATUS_SERVER_ERROR);
   }
 
  /*
@@ -130,7 +126,7 @@ cupsdCheckPolicy(cupsd_policy_t *p,	/* I - Policy */
   if ((po = cupsdFindPolicyOp(p, con->request->request.op.operation_id)) == NULL)
   {
     cupsdLogMessage(CUPSD_LOG_DEBUG2, "cupsdCheckPolicy: No matching operation, returning 0.");
-    return ((http_status_t)0);
+    return (HTTP_STATUS_OK);
   }
 
   con->best = po;
@@ -274,7 +270,7 @@ cupsdGetPrivateAttrs(
 #ifdef DEBUG
   cupsdLogMessage(CUPSD_LOG_DEBUG2,
                   "cupsdGetPrivateAttrs(policy=%p(%s), con=%p(%d), "
-		  "printer=%p(%s), owner=\"%s\")", policy, policy->name, con,
+		  "printer=%p(%s), owner=\"%s\")", policy, policy ? policy->name : "", con,
 		  con->number, printer, printer ? printer->name : "", owner);
 #endif /* DEBUG */
 
