@@ -1,24 +1,21 @@
 /*
  * Printer definitions for the CUPS scheduler.
  *
- * Copyright 2007-2016 by Apple Inc.
- * Copyright 1997-2007 by Easy Software Products, all rights reserved.
+ * Copyright © 2021-2023 by OpenPrinting.
+ * Copyright @ 2007-2017 by Apple Inc.
+ * Copyright @ 1997-2007 by Easy Software Products, all rights reserved.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more information.
  */
 
-#ifdef HAVE_DNSSD
+#ifdef HAVE_MDNSRESPONDER
 #  include <dns_sd.h>
 #elif defined(HAVE_AVAHI)
 #  include <avahi-client/client.h>
 #  include <avahi-client/publish.h>
 #  include <avahi-common/error.h>
 #  include <avahi-common/thread-watch.h>
-#endif /* HAVE_DNSSD */
+#endif /* HAVE_MDNSRESPONDER */
 #include <cups/pwg-private.h>
 
 
@@ -39,14 +36,14 @@ typedef struct
  * DNS-SD types to make the code cleaner/clearer...
  */
 
-#ifdef HAVE_DNSSD
+#ifdef HAVE_MDNSRESPONDER
 typedef DNSServiceRef cupsd_srv_t;	/* Service reference */
 typedef TXTRecordRef cupsd_txt_t;	/* TXT record */
 
 #elif defined(HAVE_AVAHI)
 typedef AvahiEntryGroup *cupsd_srv_t;	/* Service reference */
 typedef AvahiStringList *cupsd_txt_t;	/* TXT record */
-#endif /* HAVE_DNSSD */
+#endif /* HAVE_MDNSRESPONDER */
 
 
 /*
@@ -58,6 +55,7 @@ typedef struct cupsd_job_s cupsd_job_t;
 struct cupsd_printer_s
 {
   _cups_rwlock_t lock;			/* Concurrency lock for background updates */
+  int		printer_id;		/* Printer ID */
   char		*uri,			/* Printer URI */
 		*uuid,			/* Printer UUID */
 		*hostname,		/* Host printer resides on */
@@ -68,6 +66,7 @@ struct cupsd_printer_s
 		*info,			/* Description */
 		*organization,		/* Organization name */
 		*organizational_unit,	/* Organizational unit (department, etc.) */
+		*strings,		/* Strings file, if any */
 		*op_policy,		/* Operation policy name */
 		*error_policy;		/* Error policy */
   cupsd_policy_t *op_policy_ptr;	/* Pointer to operation policy */
@@ -115,17 +114,17 @@ struct cupsd_printer_s
   time_t	marker_time;		/* Last time marker attributes were updated */
   _ppd_cache_t	*pc;			/* PPD cache and mapping data */
 
-#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
+#ifdef HAVE_DNSSD
   char		*reg_name,		/* Name used for service registration */
 		*pdl;			/* pdl value for TXT record */
   cupsd_srv_t	ipp_srv;		/* IPP service(s) */
-#  ifdef HAVE_DNSSD
-#    ifdef HAVE_SSL
+#  ifdef HAVE_MDNSRESPONDER
+#    ifdef HAVE_TLS
   cupsd_srv_t	ipps_srv;		/* IPPS service(s) */
-#    endif /* HAVE_SSL */
+#    endif /* HAVE_TLS */
   cupsd_srv_t	printer_srv;		/* LPD service */
-#  endif /* HAVE_DNSSD */
-#endif /* HAVE_DNSSD || HAVE_AVAHI */
+#  endif /* HAVE_MDNSRESPONDER */
+#endif /* HAVE_DNSSD */
 };
 
 
@@ -137,6 +136,8 @@ VAR ipp_t		*CommonData	VALUE(NULL);
 					/* Common printer object attrs */
 VAR cups_array_t	*CommonDefaults	VALUE(NULL);
 					/* Common -default option names */
+VAR int			NextPrinterId	VALUE(1);
+					/* Next printer-id value */
 VAR cups_array_t	*Printers	VALUE(NULL);
 					/* Printer list */
 VAR cupsd_printer_t	*DefaultPrinter	VALUE(NULL);
@@ -156,6 +157,7 @@ extern cupsd_printer_t	*cupsdAddPrinter(const char *name);
 extern void		cupsdCreateCommonData(void);
 extern void		cupsdDeleteAllPrinters(void);
 extern int		cupsdDeletePrinter(cupsd_printer_t *p, int update);
+extern void             cupsdDeleteTemporaryPrinters(int force);
 extern cupsd_printer_t	*cupsdFindDest(const char *name);
 extern cupsd_printer_t	*cupsdFindPrinter(const char *name);
 extern cupsd_quota_t	*cupsdFindQuota(cupsd_printer_t *p,
