@@ -4,11 +4,7 @@
  * Copyright 2007-2016 by Apple Inc.
  * Copyright 2006-2007 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * file is missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more information.
  */
 
 /*
@@ -199,8 +195,7 @@ static int		cupsd_in_select = 0;
 #endif /* HAVE_EPOLL || HAVE_KQUEUE */
 
 #ifdef HAVE_KQUEUE
-static int		cupsd_kqueue_fd = -1,
-			cupsd_kqueue_changes = 0;
+static int		cupsd_kqueue_fd = -1;
 static struct kevent	*cupsd_kqueue_events = NULL;
 #elif defined(HAVE_POLL)
 static int		cupsd_alloc_pollfds = 0,
@@ -423,8 +418,6 @@ cupsdDoSelect(long timeout)		/* I - Timeout in seconds */
   else
     nfds = kevent(cupsd_kqueue_fd, NULL, 0, cupsd_kqueue_events, MaxFDs, NULL);
 
-  cupsd_kqueue_changes = 0;
-
   for (i = nfds, event = cupsd_kqueue_events; i > 0; i --, event ++)
   {
     fdptr = (_cupsd_fd_t *)event->udata;
@@ -809,7 +802,6 @@ cupsdStartSelect(void)
 
 #elif defined(HAVE_KQUEUE)
   cupsd_kqueue_fd      = kqueue();
-  cupsd_kqueue_changes = 0;
   cupsd_kqueue_events  = calloc((size_t)MaxFDs, sizeof(struct kevent));
 
 #elif defined(HAVE_POLL)
@@ -860,8 +852,6 @@ cupsdStopSelect(void)
     cupsd_kqueue_fd = -1;
   }
 
-  cupsd_kqueue_changes = 0;
-
 #elif defined(HAVE_POLL)
 #  ifdef HAVE_EPOLL
   if (cupsd_epoll_events)
