@@ -1,6 +1,7 @@
 /*
  * MIME typing routines for CUPS.
  *
+ * Copyright © 2021-2023 by OpenPrinting.
  * Copyright © 2007-2019 by Apple Inc.
  * Copyright © 1997-2006 by Easy Software Products, all rights reserved.
  *
@@ -220,7 +221,7 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
     else if (*rule == '+' && current != NULL)
     {
       if (logic != MIME_MAGIC_AND &&
-          current != NULL && current->prev != NULL)
+          current->prev != NULL)
       {
        /*
         * OK, we have more than 1 rule in the current tree level...  Make a
@@ -797,10 +798,16 @@ mime_check_rules(
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fb->fp, rules->offset);
-	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
-	                              sizeof(fb->buffer));
-	    fb->offset = rules->offset;
+            if (cupsFileSeek(fb->fp, rules->offset) < 0)
+            {
+              fb->length = 0;
+              fb->offset = 0;
+            }
+            else
+            {
+	      fb->length = cupsFileRead(fb->fp, (char *)fb->buffer, sizeof(fb->buffer));
+	      fb->offset = rules->offset;
+	    }
 
 	    DEBUG_printf(("4mime_check_rules: MIME_MAGIC_ASCII fb->length=%d", fb->length));
 	  }
@@ -841,10 +848,16 @@ mime_check_rules(
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fb->fp, rules->offset);
-	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
-	                              sizeof(fb->buffer));
-	    fb->offset = rules->offset;
+            if (cupsFileSeek(fb->fp, rules->offset) < 0)
+            {
+              fb->length = 0;
+              fb->offset = 0;
+            }
+            else
+            {
+	      fb->length = cupsFileRead(fb->fp, (char *)fb->buffer, sizeof(fb->buffer));
+	      fb->offset = rules->offset;
+	    }
 
 	    DEBUG_printf(("4mime_check_rules: MIME_MAGIC_PRINTABLE fb->length=%d", fb->length));
 	  }
@@ -890,10 +903,16 @@ mime_check_rules(
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fb->fp, rules->offset);
-	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
-	                              sizeof(fb->buffer));
-	    fb->offset = rules->offset;
+            if (cupsFileSeek(fb->fp, rules->offset) < 0)
+            {
+              fb->length = 0;
+              fb->offset = 0;
+            }
+            else
+            {
+	      fb->length = cupsFileRead(fb->fp, (char *)fb->buffer, sizeof(fb->buffer));
+	      fb->offset = rules->offset;
+	    }
 
 	    DEBUG_printf(("4mime_check_rules: MIME_MAGIC_REGEX fb->length=%d", fb->length));
 
@@ -936,10 +955,16 @@ mime_check_rules(
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fb->fp, rules->offset);
-	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
-	                              sizeof(fb->buffer));
-	    fb->offset = rules->offset;
+            if (cupsFileSeek(fb->fp, rules->offset) < 0)
+            {
+              fb->length = 0;
+              fb->offset = 0;
+            }
+            else
+            {
+	      fb->length = cupsFileRead(fb->fp, (char *)fb->buffer, sizeof(fb->buffer));
+	      fb->offset = rules->offset;
+	    }
 
 	    DEBUG_printf(("4mime_check_rules: MIME_MAGIC_STRING fb->length=%d", fb->length));
 
@@ -973,10 +998,16 @@ mime_check_rules(
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fb->fp, rules->offset);
-	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
-	                              sizeof(fb->buffer));
-	    fb->offset = rules->offset;
+            if (cupsFileSeek(fb->fp, rules->offset) < 0)
+            {
+              fb->length = 0;
+              fb->offset = 0;
+            }
+            else
+            {
+	      fb->length = cupsFileRead(fb->fp, (char *)fb->buffer, sizeof(fb->buffer));
+	      fb->offset = rules->offset;
+	    }
 
 	    DEBUG_printf(("4mime_check_rules: MIME_MAGIC_ISTRING fb->length=%d", fb->length));
 	  }
@@ -1003,10 +1034,16 @@ mime_check_rules(
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fb->fp, rules->offset);
-	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
-	                              sizeof(fb->buffer));
-	    fb->offset = rules->offset;
+            if (cupsFileSeek(fb->fp, rules->offset) < 0)
+            {
+              fb->length = 0;
+              fb->offset = 0;
+            }
+            else
+            {
+	      fb->length = cupsFileRead(fb->fp, (char *)fb->buffer, sizeof(fb->buffer));
+	      fb->offset = rules->offset;
+	    }
 
 	    DEBUG_printf(("4mime_check_rules: MIME_MAGIC_CHAR fb->length=%d", fb->length));
 	  }
@@ -1019,8 +1056,7 @@ mime_check_rules(
 	  if (fb->length < 1)
 	    result = 0;
 	  else
-	    result = (fb->buffer[rules->offset - fb->offset] ==
-	                  rules->value.charv);
+	    result = (fb->buffer[rules->offset - fb->offset] == rules->value.charv);
 	  break;
 
       case MIME_MAGIC_SHORT :
@@ -1035,10 +1071,16 @@ mime_check_rules(
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fb->fp, rules->offset);
-	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
-	                              sizeof(fb->buffer));
-	    fb->offset = rules->offset;
+            if (cupsFileSeek(fb->fp, rules->offset) < 0)
+            {
+              fb->length = 0;
+              fb->offset = 0;
+            }
+            else
+            {
+	      fb->length = cupsFileRead(fb->fp, (char *)fb->buffer, sizeof(fb->buffer));
+	      fb->offset = rules->offset;
+	    }
 
 	    DEBUG_printf(("4mime_check_rules: MIME_MAGIC_SHORT fb->length=%d", fb->length));
 	  }
@@ -1049,7 +1091,9 @@ mime_check_rules(
 	  */
 
 	  if (fb->length < 2)
+	  {
 	    result = 0;
+	  }
 	  else
 	  {
 	    bufptr = fb->buffer + rules->offset - fb->offset;
@@ -1070,10 +1114,16 @@ mime_check_rules(
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fb->fp, rules->offset);
-	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
-	                              sizeof(fb->buffer));
-	    fb->offset = rules->offset;
+            if (cupsFileSeek(fb->fp, rules->offset) < 0)
+            {
+              fb->length = 0;
+              fb->offset = 0;
+            }
+            else
+            {
+	      fb->length = cupsFileRead(fb->fp, (char *)fb->buffer, sizeof(fb->buffer));
+	      fb->offset = rules->offset;
+	    }
 
 	    DEBUG_printf(("4mime_check_rules: MIME_MAGIC_INT fb->length=%d", fb->length));
 	  }
@@ -1084,11 +1134,13 @@ mime_check_rules(
 	  */
 
 	  if (fb->length < 4)
+	  {
 	    result = 0;
+	  }
 	  else
 	  {
 	    bufptr = fb->buffer + rules->offset - fb->offset;
-	    intv   = (unsigned)((((((bufptr[0] << 8) | bufptr[1]) << 8) | bufptr[2]) << 8) | bufptr[3]);
+	    intv   = (unsigned)((bufptr[0] << 24) | (bufptr[1] << 16) | (bufptr[2] << 8) | bufptr[3]);
 	    result = (intv == rules->value.intv);
 	  }
 	  break;
@@ -1113,10 +1165,16 @@ mime_check_rules(
 	    * Reload file buffer...
 	    */
 
-            cupsFileSeek(fb->fp, rules->offset);
-	    fb->length = cupsFileRead(fb->fp, (char *)fb->buffer,
-	                              sizeof(fb->buffer));
-	    fb->offset = rules->offset;
+            if (cupsFileSeek(fb->fp, rules->offset) < 0)
+            {
+              fb->length = 0;
+              fb->offset = 0;
+            }
+            else
+            {
+	      fb->length = cupsFileRead(fb->fp, (char *)fb->buffer, sizeof(fb->buffer));
+	      fb->offset = rules->offset;
+	    }
 
 	    DEBUG_printf(("4mime_check_rules: MIME_MAGIC_CONTAINS fb->length=%d", fb->length));
 	  }
@@ -1126,19 +1184,21 @@ mime_check_rules(
 	  * short then don't compare - it can't match...
 	  */
 
-	  if ((rules->offset + rules->length) > (fb->offset + fb->length))
-	    result = 0;
-	  else
+	  result = 0;
+	  if ((rules->offset + rules->length) <= (fb->offset + fb->length))
 	  {
 	    if (fb->length > rules->region)
 	      region = rules->region - rules->length;
 	    else
 	      region = fb->length - rules->length;
 
-	    for (n = 0; n < region; n ++)
-	      if ((result = (memcmp(fb->buffer + rules->offset - fb->offset + n, rules->value.stringv, (size_t)rules->length) == 0)) != 0)
-		break;
-          }
+		  for (n = 0; n < region; n ++)
+	      if (!memcmp(fb->buffer + rules->offset - fb->offset + n, rules->value.stringv, (size_t)rules->length))
+		    {
+		      result = 1;
+		      break;
+		    }
+	  }
 	  break;
 
       default :
@@ -1157,9 +1217,9 @@ mime_check_rules(
       result = !result;
 
    /*
-    * OK, now if the current logic is OR and this result is true, the this
+    * OK, now if the current logic is OR and this result is true, this
     * rule set is true.  If the current logic is AND and this result is false,
-    * the the rule set is false...
+    * the rule set is false...
     */
 
     DEBUG_printf(("5mime_check_rules: result of test %p (MIME_MAGIC_%s) is %d",
