@@ -1,7 +1,8 @@
 /*
  * "lp" command for CUPS.
  *
- * Copyright © 2007-2019 by Apple Inc.
+ * Copyright © 2021-2023 by OpenPrinting.
+ * Copyright © 2007-2021 by Apple Inc.
  * Copyright © 1997-2007 by Easy Software Products.
  *
  * Licensed under Apache License v2.0.  See the file "LICENSE" for more
@@ -19,7 +20,7 @@
  * Local functions.
  */
 
-static int	restart_job(const char *command, int job_id);
+static int	restart_job(const char *command, int job_id, const char *job_hold_until);
 static int	set_job_attrs(const char *command, int job_id, int num_options, cups_option_t *options);
 static void	usage(void) _CUPS_NORETURN;
 
@@ -92,11 +93,11 @@ main(int  argc,				/* I - Number of command-line arguments */
         switch (*opt)
 	{
 	  case 'E' : /* Encrypt */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	      cupsSetEncryption(HTTP_ENCRYPT_REQUIRED);
 #else
 	      _cupsLangPrintf(stderr, _("%s: Sorry, no encryption support."), argv[0]);
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 	      break;
 
 	  case 'U' : /* Username */
@@ -161,6 +162,12 @@ main(int  argc,				/* I - Number of command-line arguments */
 				  "name."), argv[0]);
 		return (1);
 	      }
+	      else if (cupsLastError() == IPP_STATUS_ERROR_NOT_FOUND)
+	      {
+		_cupsLangPrintf(stderr,
+				_("%s: Error - The printer or class does not exist."), argv[0]);
+		return (1);
+	      }
 	      break;
 
 	  case 'f' : /* Form */
@@ -280,9 +287,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		return (1);
 	      }
 
-	      sprintf(buffer, "%d", num_copies);
-	      num_options = cupsAddOption("copies", buffer, num_options,
-					  &options);
+	      num_options = cupsAddIntegerOption("copies", num_copies, num_options, &options);
 	      break;
 
 	  case 'o' : /* Option */
@@ -342,9 +347,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		return (1);
 	      }
 
-	      sprintf(buffer, "%d", priority);
-	      num_options = cupsAddOption("job-priority", buffer, num_options,
-					  &options);
+	      num_options = cupsAddIntegerOption("job-priority", priority, num_options, &options);
 	      break;
 
 	  case 's' : /* Silent */
@@ -426,7 +429,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		  return (1);
 		}
 
-		if (restart_job(argv[0], job_id))
+		if (restart_job(argv[0], job_id, cupsGetOption("job-hold-until", num_options, options)))
 		  return (1);
 	      }
 	      else
@@ -652,7 +655,8 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 static int				/* O - Exit status */
 restart_job(const char *command,	/* I - Command name */
-            int        job_id)		/* I - Job ID */
+            int        job_id,		/* I - Job ID */
+            const char *job_hold_until)	/* I - "job-hold-until" value, if any */
 {
   ipp_t		*request;		/* IPP request */
   char		uri[HTTP_MAX_URI];	/* URI for job */
@@ -660,7 +664,7 @@ restart_job(const char *command,	/* I - Command name */
 
   request = ippNewRequest(IPP_RESTART_JOB);
 
-  sprintf(uri, "ipp://localhost/jobs/%d", job_id);
+  snprintf(uri, sizeof(uri), "ipp://localhost/jobs/%d", job_id);
 
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
                "job-uri", NULL, uri);
@@ -668,6 +672,9 @@ restart_job(const char *command,	/* I - Command name */
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
                "requesting-user-name", NULL, cupsUser());
 
+  if (job_hold_until)
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD, "job-hold-until", NULL, job_hold_until);
+
   ippDelete(cupsDoRequest(CUPS_HTTP_DEFAULT, request, "/jobs"));
 
   if (cupsLastError() == IPP_STATUS_ERROR_BAD_REQUEST ||
@@ -708,7 +715,7 @@ set_job_attrs(
 
   request = ippNewRequest(IPP_SET_JOB_ATTRIBUTES);
 
-  sprintf(uri, "ipp://localhost/jobs/%d", job_id);
+  snprintf(uri, sizeof(uri), "ipp://localhost/jobs/%d", job_id);
 
   ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
                "job-uri", NULL, uri);
